from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyqtSimulator.calc_conf import *
from NodeEditor.nodeeditor.node_editor_widget import NodeEditorWidget
from PyqtSimulator.calc_node_base import *
from NodeEditor.nodeeditor.node_edge import EDGE_TYPE_DIRECT, EDGE_TYPE_BEZIER
from NodeEditor.nodeeditor.node_graphics_view import MODE_EDGE_DRAG#, MODE_EDGES_REROUTING
from NodeEditor.nodeeditor.utils import dumpException

DEBUG = False
DEBUG_CONTEXT = False


class CalculatorSubWindow(NodeEditorWidget):
    def __init__(self):
        super().__init__()
        # self.setAttribute(Qt.WA_DeleteOnClose)

        self.setTitle()

        self.initNewNodeActions()

        self.scene.addHasBeenModifiedListener(self.setTitle)
        self.scene.history.addHistoryRestoredListener(self.onHistoryRestored)
        self.scene.addDragEnterListener(self.onDragEnter)
        self.scene.addDropListener(self.onDrop)
        self.scene.setNodeClassSelector(self.getNodeClassFromData)

        self._close_event_listeners = []

    def getNodeClassFromData(self, data):
        if 'op_code' not in data: return Node
        return get_class_from_opcode(data['op_code'])

    def doEvalOutputs(self):
        # eval all output nodes
        for node in self.scene.nodes:
            if node.__class__.__name__ == "CalcNode_Output":
                node.eval()

    def onHistoryRestored(self):
        self.doEvalOutputs()

    def fileLoad(self, filename):
        if super().fileLoad(filename):
            self.doEvalOutputs()
            return True

        return False

    def initNewNodeActions(self):
        self.node_actions = {}
        keys = list(CALC_NODES.keys())
        keys.sort()
        for key in keys:
            node = CALC_NODES[key]
            self.node_actions[node.op_code] = QAction(QIcon(node.icon), node.op_title)
            self.node_actions[node.op_code].setData(node.op_code)

    def initNodesContextMenu(self):
        context_menu = QMenu(self)
        keys = list(CALC_NODES.keys())
        keys.sort()
        for key in keys: context_menu.addAction(self.node_actions[key])
        return context_menu

    def setTitle(self):
        self.setWindowTitle(self.getUserFriendlyFilename())

    def addCloseEventListener(self, callback):
        self._close_event_listeners.append(callback)

    def closeEvent(self, event):
        for callback in self._close_event_listeners: callback(self, event)

    def onDragEnter(self, event):
        if event.mimeData().hasFormat(LISTBOX_MIMETYPE):
            event.acceptProposedAction()
        else:
            # print(" ... denied drag enter event")
            event.setAccepted(False)

    def onDrop(self, event):
        if event.mimeData().hasFormat(LISTBOX_MIMETYPE):
            eventData = event.mimeData().data(LISTBOX_MIMETYPE)
            dataStream = QDataStream(eventData, QIODevice.ReadOnly)
            pixmap = QPixmap()
            dataStream >> pixmap
            op_code = dataStream.readInt()
            text = dataStream.readQString()

            mouse_position = event.pos()
            scene_position = self.scene.grScene.views()[0].mapToScene(mouse_position)

            if DEBUG: print("GOT DROP: [%d] '%s'" % (op_code, text), "mouse:", mouse_position, "scene:", scene_position)

            try:
                node = get_class_from_opcode(op_code)(self.scene)
                node.setPos(scene_position.x(), scene_position.y())
                self.scene.history.storeHistory("Created node %s" % node.__class__.__name__)
            except Exception as e: dumpException(e)


            event.setDropAction(Qt.MoveAction)
            event.accept()
        else:
            # print(" ... drop ignored, not requested format '%s'" % LISTBOX_MIMETYPE)
            event.ignore()


    def contextMenuEvent(self, event):
        try:
            item = self.scene.getItemAt(event.pos())
            if DEBUG_CONTEXT: print(item)

            if type(item) == QGraphicsProxyWidget:
                item = item.widget()

            if hasattr(item, 'node') or hasattr(item, 'socket'):
                self.handleNodeContextMenu(event)
            elif hasattr(item, 'edge'):
                self.handleEdgeContextMenu(event)
            #elif item is None:
            else:
                self.handleNewNodeContextMenu(event)

            return super().contextMenuEvent(event)
        except Exception as e: dumpException(e)

    def handleNodeContextMenu(self, event):
        if DEBUG_CONTEXT: print("CONTEXT: NODE")
        context_menu = QMenu(self)
        markDirtyAct = context_menu.addAction("Mark Dirty")
        markDirtyDescendantsAct = context_menu.addAction("Mark Descendant Dirty")
        markInvalidAct = context_menu.addAction("Mark Invalid")
        unmarkInvalidAct = context_menu.addAction("Unmark Invalid")
        evalAct = context_menu.addAction("Eval")
        action = context_menu.exec_(self.mapToGlobal(event.pos()))

        selected = None
        item = self.scene.getItemAt(event.pos())
        if type(item) == QGraphicsProxyWidget:
            item = item.widget()

        if hasattr(item, 'node'):
            selected = item.node
        if hasattr(item, 'socket'):
            selected = item.socket.node

        if DEBUG_CONTEXT: print("got item:", selected)
        if selected and action == markDirtyAct: selected.markDirty()
        if selected and action == markDirtyDescendantsAct: selected.markDescendantsDirty()
        if selected and action == markInvalidAct: selected.markInvalid()
        if selected and action == unmarkInvalidAct: selected.markInvalid(False)
        if selected and action == evalAct:
            val = selected.eval()
            if DEBUG_CONTEXT: print("EVALUATED:", val)


    def handleEdgeContextMenu(self, event):
        if DEBUG_CONTEXT: print("CONTEXT: EDGE")
        context_menu = QMenu(self)
        bezierAct = context_menu.addAction("Bezier Edge")
        directAct = context_menu.addAction("Direct Edge")
        action = context_menu.exec_(self.mapToGlobal(event.pos()))

        selected = None
        item = self.scene.getItemAt(event.pos())
        if hasattr(item, 'edge'):
            selected = item.edge

        if selected and action == bezierAct: selected.edge_type = EDGE_TYPE_BEZIER
        if selected and action == directAct: selected.edge_type = EDGE_TYPE_DIRECT


    # helper functions
    def determine_target_socket_of_node(self, was_dragged_flag, new_calc_node):
        target_socket = None
        if was_dragged_flag:
            if len(new_calc_node.inputs) > 0: target_socket = new_calc_node.inputs[0]
        else:
            if len(new_calc_node.outputs) > 0: target_socket = new_calc_node.outputs[0]
        return target_socket

    def finish_new_node_state(self, new_calc_node):
        self.scene.doDeselectItems()
        new_calc_node.grNode.doSelect(True)
        new_calc_node.grNode.onSelected()


    def handleNewNodeContextMenu(self, event):

        if DEBUG_CONTEXT: print("CONTEXT: EMPTY SPACE")
        context_menu = self.initNodesContextMenu()
        action = context_menu.exec_(self.mapToGlobal(event.pos()))

        if action is not None:
            new_calc_node = get_class_from_opcode(action.data())(self.scene)
            scene_pos = self.scene.getView().mapToScene(event.pos())
            new_calc_node.setPos(scene_pos.x(), scene_pos.y())
            if DEBUG_CONTEXT: print("Selected node:", new_calc_node)

            if self.scene.getView().mode == MODE_EDGE_DRAG:
                # if we were dragging an edge...
                target_socket = self.determine_target_socket_of_node(self.scene.getView().dragging.drag_start_socket.is_output, new_calc_node)
                if target_socket is not None:
                    self.scene.getView().dragging.edgeDragEnd(target_socket.grSocket)
                    self.finish_new_node_state(new_calc_node)

            else:
                self.scene.history.storeHistory("Created %s" % new_calc_node.__class__.__name__)



# -*- coding: utf-8 -*-
"""
A module containing Graphics representation of a :class:`~nodeeditor.node_socket.Socket`
"""
from PyQt5.QtWidgets import *
from PyQt5.QtGui import *
from PyQt5.QtCore import *

SOCKET_COLORS = [
    QColor("#FFFF7700"),
    QColor("#FF52e220"),
    QColor("#FF0056a6"),
    QColor("#FFa86db1"),
    QColor("#FFb54747"),
    QColor("#FFdbe220"),
    QColor("#FF888888"),
]

class QDMGraphicsSocket(QGraphicsItem):
    """Class representing Graphic `Socket` in ``QGraphicsScene``"""
    def __init__(self, socket:'Socket'):
        """
        :param socket: reference to :class:`~nodeeditor.node_socket.Socket`
        :type socket: :class:`~nodeeditor.node_socket.Socket`
        """
        super().__init__(socket.node.grNode)

        self.socket = socket

        self.isHighlighted = False

        self.radius = 6.0
        self.outline_width = 1.0
        self.initAssets()

    @property
    def socket_type(self):
        return self.socket.socket_type

    def getSocketColor(self, key):
        """Returns the ``QColor`` for this ``key``"""
        if type(key) == int: return SOCKET_COLORS[key]
        elif type(key) == str: return QColor(key)
        return Qt.transparent

    def changeSocketType(self):
        """Change the Socket Type"""
        self._color_background = self.getSocketColor(self.socket_type)
        self._brush = QBrush(self._color_background)
        # print("Socket changed to:", self._color_background.getRgbF())
        self.update()

    def initAssets(self):
        """Initialize ``QObjects`` like ``QColor``, ``QPen`` and ``QBrush``"""

        # determine socket color
        self._color_background = self.getSocketColor(self.socket_type)
        self._color_outline = QColor("#FF000000")
        self._color_highlight = QColor("#FF37A6FF")

        self._pen = QPen(self._color_outline)
        self._pen.setWidthF(self.outline_width)
        self._pen_highlight = QPen(self._color_highlight)
        self._pen_highlight.setWidthF(2.0)
        self._brush = QBrush(self._color_background)

    def paint(self, painter, QStyleOptionGraphicsItem, widget=None):
        """Painting a circle"""
        painter.setBrush(self._brush)
        painter.setPen(self._pen if not self.isHighlighted else self._pen_highlight)
        #painter.drawEllipse(-self.radius, -self.radius, 2 * self.radius, 2 * self.radius)
        painter.drawEllipse(int(-self.radius), int(-self.radius), int(2 * self.radius), int(2 * self.radius))

    def boundingRect(self) -> QRectF:
        """Defining Qt' bounding rectangle"""
        return QRectF(
            - self.radius - self.outline_width,
            - self.radius - self.outline_width,
            2 * (self.radius + self.outline_width),
            2 * (self.radius + self.outline_width),
        )


from PyQt5.QtCore import *
from PyqtSimulator.calc_conf import *
from PyqtSimulator.calc_node_base import *
from NodeEditor.nodeeditor.utils import dumpException

#############les modèles d'un groupe frigorifique#################



@register_node(OP_NODE_AIRMIX)
class CalcNode_AIRMIX(CalcNode):
    icon = "icons/mixer.png"
    op_code = OP_NODE_AIRMIX
    op_title = "Mélangeur d'air"
    content_label = "+"
    content_label_objname = "calc_node_bg"
    
 

    def evalOperation(self, input1, input2):
        self.value=[]
        
        m_as1=input1[1]/(1+(input1[0]/1000)) #kg/s
        m_as2=input2[1]/(1+(input2[0]/1000)) #kg/s
       
        self.value.append((m_as1*input1[0]+m_as2*input2[0])/(m_as1+m_as2)) #Humidité absolue
        self.value.append(input1[1]+input2[1]) #débit kg/s OK
        self.value.append(min(input1[2],input2[2])) #pression min - OK
        self.value.append((m_as1*input1[3]+m_as2*input2[3])/(m_as1+m_as2)) #Enthalpie kJ/kas
            
        return self.value
    



# -*- coding: utf-8 -*-
"""
Created on Sat Sep  5 19:39:26 2020

@author: zohei
"""

from PyQt5.QtCore import *
from PyqtSimulator.calc_conf import *
from PyqtSimulator.calc_node_base import *
from NodeEditor.nodeeditor.utils import dumpException

#############les modèles d'un groupe frigorifique#################
#from AHU import FreshAir
from AHU import CoolingCoil_Sensible
#from AHU.Humidification import Humidifier


from AHU.Connect import Air_connect
from AHU.AirPort.AirPort import AirPort




class CalcCoolingCoilSensibleContent(QDMNodeContentWidget):
    def initUI(self):
        
        self.P_drop_lbl=QLabel("Perte de pression (bar)", self) 
        self.P_drop_edit=QLineEdit("0.001", self)
        
        self.T_target_lbl=QLabel("Temp. cible (°C)", self) 
        self.T_target_edit=QLineEdit("20", self)
        
        
        
        # self.Tref_lbl=QLabel("refoidissement cible (°C)", self) 
        # self.Tref_edit=QLineEdit("80", self)
        
        self.HA_lbl_title = QLabel("Qth(kW)", self)
        self.HA_lbl = QLabel("", self)
        
        # self.Qlosses_lbl_title = QLabel("Energie dissipée (kW):", self)
        # self.Qlosses_lbl = QLabel("", self)
        
        # self.Tis_lbl_title = QLabel("Temp. isentrop. (°C)", self)
        # self.Tis_lbl = QLabel("", self)
        
        
        
        
         
        self.layout=QVBoxLayout()

        self.layout.addWidget(self.P_drop_lbl)
        self.layout.addWidget(self.P_drop_edit) 
        
        # self.layout.addWidget(self.Tref_lbl)
        # self.layout.addWidget(self.Tref_edit)
        
        self.layout.addWidget(self.T_target_lbl)
        self.layout.addWidget(self.T_target_edit)
        
        
        
        self.layout.addWidget(self.HA_lbl_title)
        self.layout.addWidget(self.HA_lbl)  
        
        # self.layout.addWidget(self.Qlosses_lbl_title)
        # self.layout.addWidget(self.Qlosses_lbl) 
      
        # self.layout.addWidget(self.Tis_lbl_title)
        # self.layout.addWidget(self.Tis_lbl)
        
     
        
                         
        
        self.setLayout(self.layout)
        
        self.layout.setAlignment(Qt.AlignRight)
        self.layout.setObjectName(self.node.content_label_objname)
        
    def serialize(self):
        res = super().serialize()
        res['value'] = self.P_drop_edit.text()
        res2 = super().serialize()
        res2['value2'] = self.T_target_edit.text()
        
        res3 = super().serialize()
       # res3['value3'] = self.Tref_edit.text()
        
        
        
        # res4 = super().serialize()
        # res4['value4'] = self.F_kgs_edit.text()
        
        return res,res2 #,res3 #,res4

    def deserialize(self, data, hashmap={}):
        res = super().deserialize(data, hashmap)
        res2 = super().deserialize(data, hashmap)
       # res3 = super().deserialize(data, hashmap)
        # res4 = super().deserialize(data, hashmap)
        # print("res=",res,res2,res3,res4)
        # print("dataaaaaaaaaa=",data)
        try:
            
            value = data[0]["value"]
            value2 = data[1]['value2']
           # value3 = data[2]['value3']
           # value4 = data[3]['value4']
            
            # print("values=",value,value2,value3,value4)
            
            self.P_drop_edit.setText(value)
            self.T_target_edit.setText(value2)
            #self.Tref_edit.setText(value3)
            # self.F_kgs_edit.setText(value4)
            
            return True & res  & res2 #& res3 #& res4
        except Exception as e:
            dumpException(e)
        return res ,res2,res3 #,res4

@register_node(OP_NODE_COOLING_COIL_SENSIBLE)
class CalcNode_cooling_coil_sensible(CalcNode):
    icon = "icons/cooling_coil.png"
    op_code = OP_NODE_COOLING_COIL_SENSIBLE
    op_title = "Cooling Coil Sensible"
    content_label = "/"
    content_label_objname = "calc_node_cooling_coil_sensible"
    
    def __init__(self, scene):
        super().__init__(scene, inputs=[2], outputs=[1])
        self.eval()
        
    def initInnerClasses(self):
        self.content = CalcCoolingCoilSensibleContent(self)
        self.grNode = CalcGraphicsNode(self)
        
        
        self.grNode.height=250
        self.grNode.width=200
        
        self.content.P_drop_edit.textChanged.connect(self.onInputChanged)
#        self.content.Tref_edit.textChanged.connect(self.onInputChanged)
        self.content.T_target_edit.textChanged.connect(self.onInputChanged)
  
    def evalOperation(self, input1, input2):
        self.value=[]
        
        a=AirPort()
        a.w=input1[0]
        a.F=input1[1]
        a.P=input1[2]*1e5
        a.h=input1[3]
        
        COOLING_COIL_SENSIBLE=CoolingCoil_Sensible.Object()
        Air_connect(COOLING_COIL_SENSIBLE.Inlet,a)
        ################""""
        u_P_drop = self.content.P_drop_edit.text()
        s_P_drop = float(u_P_drop)
        ####################
        COOLING_COIL_SENSIBLE.P_drop=1e5*s_P_drop
        COOLING_COIL_SENSIBLE.To_target=float(self.content.T_target_edit.text()) 
        #HEATING_COIL.Tdischarge_target=float(self.content.Tref_edit.text())
        COOLING_COIL_SENSIBLE.calculate()
        
       
        self.value.append(COOLING_COIL_SENSIBLE.Outlet.w) #HAe
        self.value.append(COOLING_COIL_SENSIBLE.Outlet.F) #débit
        self.value.append(COOLING_COIL_SENSIBLE.Outlet.P/1e5) #pression min
        self.value.append(COOLING_COIL_SENSIBLE.Outlet.h) #Enthalpie
        
        self.content.HA_lbl.setText("%f" % (COOLING_COIL_SENSIBLE.Q_th)) #"%d" % 
        #self.content.Qlosses_lbl.setText("%f" % (COMP.Q_losses/1000))
        #self.content.Tis_lbl.setText("%f" % (HEATING_COIL.Tis-273.15))
        #self.content.lbl.setText("%f" % val[3])
        
        
            
        return self.value


from AHU.air_humide import air_humide
from AHU.air_humide import air_humide_NB
from AHU.AirPort.AirPort import AirPort


from scipy import *
from pylab import *
from scipy.optimize import bisect   
from scipy.optimize import fsolve   


class Object:
    def __init__(self):
        self.Inlet=AirPort() 
        self.Outlet=AirPort()
        
        #parameter :
        self.P_drop=0
        
        '''Point de consignes'''
        self.w_target=8 
        self.T_target=15
        self.Outlet_RH=90
        
        
        
        '''Température point de rosée de l air au contact de la Coil froide '''
        self.T_sat=7  
        '''Températures inlet et outlet'''
        self.T_inlet=0
        self.T_Outlet=12
        self.Outlet_Pvsat=100# initialisation
        self.Inlet_RH=0
        
       
                
        """facteur de bypass"""
        self.FB=0.2
        self.w_sat=0
        self.h_sat=0
        # calcul Outlet Coil
       
        self.Q_th=12      
        self.Eff=0.8 #efficacité de la Coil froide
        
        self.F=0
        self.F_m3h=0
        self.F_dry=0 #Débit d'air sec
        
    def calculate(self):
        #print("self.Inlet.P=",self.Inlet.P,"air_humide.Air_Pv_sat(self.T_sat)=",air_humide.Air_Pv_sat(self.T_sat),"self.T_sat=",self.T_sat)
        self.w_sat=air_humide.Air_w(T_db=self.T_sat, RH=100, P=self.Inlet.P)
        self.h_sat=air_humide.Air_h(T_db=self.T_sat,w=self.w_sat)
        
        self.T_inlet=air_humide_NB.Air3_Tdb(self.Inlet.w/1000, self.Inlet.P, self.Inlet.h)
        self.F=self.Inlet.F
        #print("self.w_sat=",self.w_sat,"self.w_target=",self.w_target)
        
        
      #  Test du calcul de point de Outlet à 90% d'humidité 7.5 g/kg, 12°C
        self.Outlet.w=self.w_target
        # print("CC.Outlet.w consigne",self.Outlet.w)
        self.Inlet_RH=air_humide.Air_RH(Pv_sat=air_humide.Air_Pv_sat(self.T_inlet),w=self.Inlet.w,P=self.Inlet.P)
        
      
        if self.Inlet.w>=self.w_target and self.w_target>=self.w_sat: 
            
            if self.Inlet_RH<=90 :
        
                def syst(var): # définition du système
                    self.Outlet_Pvsat,self.T_Outlet, = var[0], var[1] # définition des variables
                    eq1 =self.Outlet_RH-air_humide.Air_RH(Pv_sat=self.Outlet_Pvsat,w=self.w_target,P=self.Inlet.P)  #calcul de Pvsat
                    eq2 =self.Outlet_Pvsat-air_humide.Air_Pv_sat(self.T_Outlet) #calcul de T      
                    #eq2=self.T_Outlet-12          
                    #eq3 =self.Outlet.w-air_humide.w(self.Outlet_Pvsat, self.Outlet_RH, self.Inlet.P) #recalcul de w
                    res = [eq1,eq2]
                    return res
                        #self.ho=self.hi
                x0,y0 = 1341,12 # Initialisation de la recherche des solutions numériques
                sol_ini = [x0,y0]
                
                
                # print(self.Outlet_Pvsat)
        
                x=fsolve(syst, sol_ini)
                # print("Résultats x=",x)
          
                # #self.T_Outlet=12
            
            
                # print("CC.Outlet.w recalculée",self.Outlet.w)
             
                               
                # print("self.Outlet_RH=",self.Outlet_RH)
                # print("self.Outlet.w=",self.Outlet.w)
                
                # print("self.T_Outlet=",self.T_Outlet)
                # # self.Outlet.h=air_humide_NB.Air4_Hs(self.T_Outlet, self.Inlet.P, self.Outlet.w/1000)
                # print("self.Outlet.h NB=",self.Outlet.w)
                self.Outlet.h=air_humide.Air_h(T_db=self.T_Outlet, w=self.Outlet.w)
                # print("self.Outlet.h ZHA=",self.Outlet.h)   
        
            else:
                def syst(var): # définition du système
                    self.Outlet_Pvsat,self.T_Outlet, = var[0], var[1] # définition des variables
                    eq1 =100-air_humide.Air_RH(Pv_sat=self.Outlet_Pvsat,w=self.w_target,P=self.Inlet.P)  #calcul de Pvsat
                    eq2 =self.Outlet_Pvsat-air_humide.Air_Pv_sat(self.T_Outlet) #calcul de T      
                    #eq2=self.T_Outlet-12          
                    #eq3 =self.Outlet.w-air_humide.w(self.Outlet_Pvsat, self.Outlet_RH, self.Inlet.P) #recalcul de w
                    res = [eq1,eq2]
                    return res
                        #self.ho=self.hi
                x0,y0 = 1341,12 # Initialisation de la recherche des solutions numériques
                sol_ini = [x0,y0]
                
                
                # print(self.Outlet_Pvsat)
        
                x=fsolve(syst, sol_ini)
                # print("Résultats x=",x)
          
                # #self.T_Outlet=12
            
            
                # print("CC.Outlet.w recalculée",self.Outlet.w)
             
                               
                # print("self.Outlet_RH=",self.Outlet_RH)
                # print("self.Outlet.w=",self.Outlet.w)
                
                # print("self.T_Outlet=",self.T_Outlet)
                # # self.Outlet.h=air_humide_NB.Air4_Hs(self.T_Outlet, self.Inlet.P, self.Outlet.w/1000)
                # print("self.Outlet.h NB=",self.Outlet.w)
                self.Outlet.h=air_humide.Air_h(T_db=self.T_Outlet, w=self.Outlet.w)
                # print("self.Outlet.h ZHA=",self.Outlet.h) 
            
        
        # '''Cas où l'w de l'air à traiter est inférieure l'w de consigne'''
        #         ''' Avec refroidissement sensible sans déshu '''
        elif self.Inlet.w<=self.w_target and self.T_inlet>=self.T_target:
            self.Eff=(self.T_sat-self.T_target)/(self.T_inlet-self.T_sat)
            self.FB=1-self.Eff
            self.Outlet.h=air_humide_NB.Air4_Hs(self.T_target, self.Inlet.P, self.Inlet.w/1000)
            self.Outlet.w=self.Inlet.w
        
        
         # ''' sans aucune action de la Coil froide (Pas de traitement à faire) '''
        else:
            self.Outlet.h=self.Inlet.h
            self.Outlet.w=self.Inlet.w
            self.Eff=0
            self.FB=1-self.Eff
            # print('CAS 5')
            
              
       
        # '''Test que la consigne de déshu est située entre w de l'air humide et HA_sat à la surface de la Coil froide'''
        # # if self.Déshutype="Droite_sat":
        # if self.Inlet.w>=self.w_target and self.w_target>=self.w_sat:
          
        #     self.Eff=(self.Inlet.w-self.Outlet.w)/(self.Inlet.w-self.w_sat)
        #     self.FB=1-self.Eff
        #     self.Outlet.h=self.Inlet.h-self.Eff*(self.Inlet.h-self.h_sat)
        #     self.T_Outlet=air_humide_NB.Air3_Tdb(self.w_target/1000, self.Inlet.P, self.Outlet.h)
            
        #     # print('CAS 1')
            
        #     #'''cas où après la transormation, la température de l'air traité est supérieure à la température de consigne'''
           
        #     # if self.T_Outlet>=self.T_target:
        #     #     self.Inlet.h=self.Outlet.h
        #     #     self.T_inlet=self.T_Outlet
        #     #     self.Eff=(self.T_target-self.T_sat)/(self.T_inlet-self.T_sat)
        #     #     self.FB=1-self.Eff
        #     #     self.Outlet.h=self.h_sat+self.Eff*(self.Inlet.h-self.h_sat)
        #     #     self.Outlet.w=self.w_sat+self.Eff*(self.w_target-self.w_sat)
              
        #         # print('CAS 2')
                
                
        #      #Air exterieur compris entre HA_sat et HA_Target et T_air extérieur > T_target   
       
        # elif self.Inlet.w>self.w_sat and self.Inlet.w<=self.w_target and self.T_inlet>=self.T_target:
        #     self.Eff=(self.T_target-self.T_sat)/(self.T_inlet-self.T_sat)
        #     self.FB=1-self.Eff
        #     self.Outlet.h=self.h_sat+self.Eff*(self.Inlet.h-self.h_sat)
        #     self.Outlet.w=self.w_sat+self.Eff*(self.w_target-self.w_sat)
        #     #self.Outlet.h=air_humide_NB.Air4_Hs(self.T_target, self.Inlet.P, self.Inlet.w/1000)
        #     #self.Outlet.w=self.Inlet.w
        #     # print('CAS 3')
            
        # # '''Cas où l'w de l'air à traiter est inférieure à l'HA_sat à la surface de la Coil'''
        # #         ''' Avec refroidissement sensible sans déshu '''
        # elif self.Inlet.w<=self.w_sat and self.T_inlet>=self.T_target:
        #     self.Eff=(self.T_sat-self.T_target)/(self.T_inlet-self.T_sat)
        #     self.FB=1-self.Eff
        #     self.Outlet.h=air_humide_NB.Air4_Hs(self.T_target, self.Inlet.P, self.Inlet.w/1000)
        #     self.Outlet.w=self.Inlet.w
        #     #print('CAS 4')
            
        #     ''' sans aucune action de la Coil froide (Pas de traitement à faire) '''
        # else:
        #     self.Outlet.h=self.Inlet.h
        #     self.Outlet.w=self.Inlet.w
        #     self.Eff=0
        #     self.FB=1-self.Eff
        #     # print('CAS 5') 
        
        
        
        
        
        self.Outlet.P=self.Inlet.P-self.P_drop
        # print("self.Outlet.P",self.Outlet.P)  
         
         
        #self.F_dry=(self.Inlet.F)/air_humide_NB.Air3_Vs(self.Inlet.w/1000,self.Inlet.P,self.Inlet.h) #[m3/s] / [m3/kg air sec]  = [kg air sec/s]
        self.F_dry=(self.Inlet.F)/(1+(self.Inlet.w/1000))
        #print("self.F_dry=",self.F_dry)
        self.Q_th=(self.Outlet.h-self.Inlet.h)*self.F_dry       
        #self.Outlet.F=self.F_dry*air_humide_NB.Air3_Vs(self.Outlet.w/1000,self.Outlet.P,self.Outlet.h) #[kg air sec/s] * [m3/kg air sec] =[m3/s]
        self.Outlet.F=self.F_dry*(1+(self.Outlet.w/1000))
        #print("self.Inlet.F=",self.Inlet.F)
        #print("self.Outlet.F=",self.Outlet.F)
        self.Outlet.F_dry=self.F_dry
        
        
 
           
            
            





from AHU.air_humide import air_humide
from AHU.AirPort.AirPort import AirPort
import pandas as pd


class Object:
    def __init__(self):
        
        self.Inlet1=AirPort() 
        self.Inlet2=AirPort() 
        self.Outlet=AirPort()
        self.id=1
        
        self.F_dry=None
        self.F_dry1=None
        self.F_dry2=None
        self.df = None  # DataFrame pour afficher les résultats
        
        
    def calculate(self):
        
        if (self.Inlet1.F is not None and self.Inlet1.w is not None):
            self.F_dry1=self.Inlet1.F/(1+self.Inlet1.w/1000)
        
        if (self.Inlet2.F is not None and self.Inlet2.w is not None):
            self.F_dry2=self.Inlet2.F/(1+self.Inlet2.w/1000)

        #connecteur   
      
    
        #Calcul le poid d'eau du mélange si les deux flux sont connus
        if (self.F_dry1 is not None and self.F_dry2 is not None):
            self.Outlet.w=(self.Inlet1.w*self.F_dry1+self.Inlet2.w*self.F_dry2)/(self.F_dry1+self.F_dry2)
            self.Outlet.h=(self.Inlet1.h*self.F_dry1+self.Inlet2.h*self.F_dry2)/(self.F_dry1+self.F_dry2)
            self.Outlet.F=self.Inlet1.F+self.Inlet2.F 

        if (self.F_dry1 is None and self.F_dry2 is not None):
            self.Outlet.w=(self.Inlet2.w*self.F_dry2)/(self.F_dry2)
            self.Outlet.h=(self.Inlet2.h*self.F_dry2)/(self.F_dry2)
            self.Outlet.F=self.Inlet2.F 
        
        if (self.F_dry1 is not None and self.F_dry2 is None):
            self.Outlet.w=(self.Inlet1.w*self.F_dry1)/(self.F_dry1)
            self.Outlet.h=(self.Inlet1.h*self.F_dry1)/(self.F_dry1)
            self.Outlet.F=self.Inlet1.F

        
        
        self.Outlet.P=min(self.Inlet1.P,self.Inlet2.P)
        
        
        #self.T_outlet=air_humide_NB.Air3_Tdb(self.Outlet.w/1000,self.Outlet.P,self.Outlet.h)
        
        self.F_dry=(self.Outlet.F)/(1+self.Outlet.w/1000)
        self.Outlet.F_dry=self.F_dry
    
        data = {
            'ID': [self.id],  # Identifiant de l'objet
            'self.Outlet.T (C)': [self.Outlet.T],  # Température de l'air
            'self.Outlet.RH (%)': [self.Outlet.RH],  # Humidité relative
            'self.Outlet.F (kg/s)': [self.Outlet.F],  # Débit d'air en kg/s
            'self.Outlet.F_dry (kg/s)': [self.Outlet.F_dry],  # Débit d'air sec en kg/s
            'self.Outlet.P (Pa)': [self.Outlet.P],  # Pression de l'air
            'self.Outlet.h (kJ/kg)': [self.Outlet.h],  # Enthalpie spécifique
            'self.Outlet.w (g/kgdry)': [self.Outlet.w],  # Humidité absolue
            'self.Outlet.Pv_sat (Pa)': [self.Outlet.Pv_sat],  # Pression de vapeur saturante
        }

        # Convertir les données en DataFrame et transposer pour avoir les paramètres en lignes
        self.df = pd.DataFrame(data).T






from ThermodynamicCycles.FluidPort.FluidPort import FluidPort
from CoolProp.CoolProp import PropsSI
import pandas as pd 
import math

class Object :
    def __init__(self):

        self.Timestamp=None

        self.Inlet=FluidPort() 
        self.Outlet=FluidPort()
        self.df=[]
        self.Qstr_J=None #Energie stockée
        self.cumul_Qstr_kWh=0

        self.V=1
        self.Tinit_degC=12
        self.rho=1000
        self.m=1000/3600
        self.t=3600
        self.Ti_degC=60
        self.T=None
        self.T_degC=None
        self.Cp=4181
        self.U=0 #coeffcient d'échange global
        self.S=3 # surface d'échange vers l'ambiance
        self.Tamb_degC=12 #Température ambiance
        self.const=None #constante de calcul (pour simplfier l'écriture)
        self.const_t=None # constante de calcul exp
    def calculate(self):
        self.m=self.Inlet.F
        self.Ti_degC=-273.15+PropsSI('T','P',self.Inlet.P,'H',self.Inlet.h,self.Inlet.fluid)
        

        self.const=(self.m*self.Cp*self.Ti_degC+self.U*self.S*self.Tamb_degC)/(self.m*self.Cp+self.U*self.S)
        #print("Ti_degC",self.const)
        self.const_t=(self.m*self.Cp+self.U*self.S)/(self.rho*self.V*self.Cp)

        self.T_degC=self.const+(self.Tinit_degC-self.const)*math.exp(-(self.const_t*self.t))
        self.T=self.T_degC+273.17

        #recalcul du CP
        self.Cp=PropsSI('CPMASS','P',self.Inlet.P,'T',self.T,self.Inlet.fluid)
        #print("Cp=",self.Cp)
        #calcul de l'énergie stockée
        self.Qstr_J=self.rho*self.V*self.Cp*(self.T_degC-self.Tinit_degC)
        self.Qstr_kWh=self.Qstr_J/(3600000)
        self.cumul_Qstr_kWh=self.Qstr_J/(3600000)+self.cumul_Qstr_kWh
        
        self.Qstr_kW=self.Qstr_kWh/(self.t/3600)
        #connecteur de Outlet
        self.Outlet.fluid=self.Inlet.fluid
        self.Outlet.h=PropsSI('H','P',self.Inlet.P,'T',self.T,self.Inlet.fluid)
        self.Outlet.P=self.Inlet.P
        self.Outlet.F=self.Inlet.F

        

        self.df = pd.DataFrame({'MixedStorage': [self.Timestamp,self.Inlet.fluid,self.Ti_degC,self.Inlet.F,self.T_degC,self.Inlet.h,self.Outlet.h,self.Qstr_kWh,self.Qstr_kW,self.cumul_Qstr_kWh,], },
                      index = ['Timestamp','str_fluid','str_Ti_degC','str_Inlet.F','str_T_degC','str_Inlet.h','str_Outlet.h','Qstr_kWh','Qstr_kW',"cumul_Qstr_kWh",  ])

        #reinitialisation de la Température pour une n éme ittération 
        self.Tinit_degC=self.T_degC
       


from ThermodynamicCycles.FluidPort.FluidPort import FluidPort
from CoolProp.CoolProp import PropsSI
import pandas as pd 
from datetime import datetime

class Object:
    def __init__(self):
        self.Timestamp=datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        #Input and Output Connector
       # self.Inlet=FluidPort() 
        self.Outlet=FluidPort()

        #Input Data
        self.fluid=None #"air"
        self.Ti_degC=None #15 #♣°C
        self.Pi_bar=None #1.01325
        self.F=None
        
        # Callback pour adapter le débit au réseau en aval
        self.Outlet.callback = self.on_flow_change
        
        #(ISO 2533) : 288.15K (15°C) / 1.01325 bar
        self.F_Sm3s=None
        self.F_Sm3h=None


        self.F_m3s=None
        
        self.F_m3h=None
        self.F_kgh=None

        # (DIN 1343) : 273.15K (0°C) / 1.01325 bar
        self.F_Nm3s=None
        self.F_Nm3h=None

     
        
        

        #Initial Values
        
        
        self.Q=0
        self.fluid_quality="liquid"
        self.df = pd.DataFrame()
    
    def on_flow_change(self):
        """
        Callback appelé lorsque self.Outlet.F change (débit demandé par le réseau en aval).
        Adapte automatiquement le débit de la source pour correspondre au réseau.
        """
        if hasattr(self, '_calculating'):
            return  # Éviter les boucles infinies pendant le calcul
        
        # Si le débit de sortie a changé (imposé par la pompe en aval), adapter
        if self.Outlet.F is not None and self.Outlet.F != self.F:
            print(f"🔄 SOURCE CALLBACK: Adaptation débit: {self.F:.3f} → {self.Outlet.F:.3f} kg/s")
            self._calculating_inverse = True
            try:
                self.F = self.Outlet.F
                # Recalculer les débits volumiques avec le nouveau débit massique
                if self.Pi_bar is not None and self.Ti_degC is not None and self.fluid is not None:
                    rho = PropsSI("D", "P", 100000*self.Pi_bar, "T", (self.Ti_degC+273.15), self.fluid)
                    self.F_m3h = self.F * 3600 / rho
                    print(f"   Nouveau débit volumique: {self.F_m3h:.2f} m³/h")
            finally:
                if hasattr(self, '_calculating_inverse'):
                    delattr(self, '_calculating_inverse')
       
    
        
    def calculate (self):
        # Marquer le début du calcul pour éviter les callbacks récursifs
        self._calculating = True
        
        try:
            if self.F_Sm3h is not None:
                self.F=self.F_Sm3h/3600*PropsSI("D", "P", 100000*1.01325, "T", (15+273.15), self.fluid)

            if self.F_Nm3h is not None:
                self.F=self.F_Nm3h/3600*PropsSI("D", "P", 100000*1.01325, "T", (0+273.15), self.fluid)
        
            if self.F_m3s is not None:
                self.F=self.F_m3s*PropsSI("D", "P", 100000*self.Pi_bar, "T", (self.Ti_degC+273.15), self.fluid)

            if self.F_m3h is not None:
                self.F=self.F_m3h/3600*PropsSI("D", "P", 100000*self.Pi_bar, "T", (self.Ti_degC+273.15), self.fluid)

            if self.F_kgh is not None:
                self.F=self.F_kgh/3600
        
            if self.F_Sm3s is not None:
                self.F=self.F_Sm3s*PropsSI("D", "P", 100000*1.01325, "T", (15+273.15), self.fluid)
        


            if self.F_Nm3s is not None:
                self.F=self.F_Nm3s*PropsSI("D", "P", 100000*1.01325, "T", (0+273.15), self.fluid)

            #print("self.F",self.F)
            if self.F is None:
                self.F=0


            self.Outlet.fluid=self.fluid
            
            # Temporairement désactiver le callback pendant la mise à jour
            temp_callback = self.Outlet.callback
            self.Outlet.callback = None
            
            self.Outlet.F=self.F
            self.Outlet.P=self.Pi_bar*100000
            
            # Réactiver le callback
            self.Outlet.callback = temp_callback

            #Inlet temperature calculation
            self.Outlet.h=PropsSI('H','P',self.Outlet.P,'T',self.Ti_degC+273.15,self.fluid)
            #print("sink h",self.Outlet.h)
            #calcul de l'état du fluid
            try:
                self.Ti_degC=PropsSI("T", "P", 100000*self.Pi_bar, "H", self.Outlet.h,self.fluid)-273.15
            except:
                self.Ti_degC=0-273.15

            try:
                pressure_critical = PropsSI("Pcrit", self.fluid)
                #print(pressure_critical)
            except:
                pass


            if (100000*self.Pi_bar)<PropsSI("Pcrit",self.fluid): #comparer à la pression critique
                Hv=PropsSI("H", "P", 100000*self.Pi_bar, "Q", 1, self.fluid)
               # print("Hv=",Hv)
                Hl=PropsSI("H", "P", 100000*self.Pi_bar, "Q", 0, self.fluid)
              #  print("Hl=",Hl)
                self.Q=1-((Hv-self.Outlet.h)/(Hv-Hl))
            
                if self.Q>=1:
                   # print(self.fluid+" vapeur"+"%.2f" % self.Q) #"%d" % 
                    self.fluid_quality="vapor"
                elif (self.Q<1 and self.Q>0):
                   # print(self.fluid+" diphasique"+"%.2f" % self.Q) #"%d" % 
                    self.fluid_quality="two-phase"
                else:
                    #print(self.fluid+" liquide"+"%.2f" % self.Q) #"%d" % 
                    self.fluid_quality="liquid"
               # print("Qualité=",self.Q)
            
            else:
                #print(self.fluid+" supercritique") #"%d" % 
                self.fluid_quality="supercritical"
            
         
            if (100000*self.Pi_bar)<PropsSI("Pcrit",self.fluid): #comparer à la pression critique
                if self.Q>1:
                    self.F_m3s=self.Outlet.F/PropsSI("D", "P", 100000*self.Pi_bar, "T", (self.Ti_degC+273.15), self.fluid)
                elif (self.Q<=1 and self.Q>=0):
                    self.F_m3s=self.Q*self.Outlet.F/PropsSI("D", "P", 100000*self.Pi_bar, "Q", 1, self.fluid)+(1-self.Q)*self.Outlet.F/PropsSI("D", "P", 100000*self.Pi_bar, "Q", 0, self.fluid)
                else:
                    self.F_m3s=self.Outlet.F/PropsSI("D", "P", 100000*self.Pi_bar, "T", (self.Ti_degC+273.15), self.fluid)        
            else:    
                self.F_m3s=self.Outlet.F/PropsSI("D", "P", 100000*self.Pi_bar, "T", (self.Ti_degC+273.15), self.fluid)
        
            self.F_Sm3s=self.Outlet.F/PropsSI("D", "P", 100000*1.01325, "T", (15+273.15), self.fluid)
            try:
                self.F_Nm3s=self.Outlet.F/PropsSI("D", "P", 100000*1.01325, "T", (0+273.15), self.fluid)
                self.F_Nm3h=self.F_Nm3s*3600
            except:
                self.F_Nm3s=None
                self.F_Nm3h=None


            self.F_Sm3h=self.F_Sm3s*3600
        

            self.F_m3h=self.F_m3s*3600
            self.F_kgh=self.Outlet.F*3600


            self.Outlet.calculate_properties()

            self.df = pd.DataFrame({'Source': [self.Timestamp,self.Outlet.fluid,round(self.Ti_degC,1),round(self.Pi_bar,2),round(self.F_Sm3h,1),self.F_Nm3h,round(self.F_m3h,1),round(self.F_kgh,3),round(self.F,3),round(self.F_m3s,3),round(self.F_Sm3s,3),self.Outlet.h], },
                      index = ['Timestamp','fluid','Ti_degC','Pi_bar','F_Sm3h','F_Nm3h','F_m3h', 'F_kgh','F_kgs','F_m3s','F_Sm3s','self.Outlet.h'])

            #réinitialiser les débits
            self.F_Sm3s=None
            self.F_Nm3s=None

            self.F_m3s=None
            self.F_Sm3h=None
            self.F_Nm3h=None
            self.F_m3h=None
            self.F_kgh=None
        
        finally:
            # Retirer le flag de calcul
            if hasattr(self, '_calculating'):
                delattr(self, '_calculating')
        
 



# Importation de la bibliothèque Cantera pour les simulations thermodynamiques
import cantera as ct
# Importation de CoolProp pour les propriétés des fluides
from CoolProp.CoolProp import PropsSI
# Importation de pandas pour la manipulation de données
import pandas as pd
# Importation de datetime pour enregistrer les horodatages
from datetime import datetime
# Importation de la classe FluidPort pour gérer les entrées et sorties de fluides
from ThermodynamicCycles.FluidPort.FluidPort import FluidPort

# Définition de la classe 'Object'
class Object:
    def __init__(self):
        # Initialisation des ports pour le carburant et l'oxydant
        self.fuel_Inlet = FluidPort()
        self.oxidizer_Inlet = FluidPort()

        # Enregistrement de l'horodatage actuel
        self.Timestamp = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        # Initialisation de l'objet gaz Cantera avec le mécanisme de réaction 'gri30.yaml'
        self.gas = ct.Solution("gri30.yaml")
        # Définition des noms par défaut du carburant et de l'oxydant
        self.fuel_name = "H2"
        self.oxidizer_name = "O2:1.0"

        # Définition du débit de carburant par défaut en kg/s
        #self.fuel_F_kgs = 0.001
        # Variable pour la quantité de chaleur de combustion (sera calculée)
        self.Q_comb = None

        # Ratio du mélange carburant/oxydant (phi) et excès d'air (à calculer)
        self.phi = None
        self.AIR_EXCESS = None
        self.products_O2_molRatio = None

        # Débit d'oxydant par défaut en kg/s
        self.oxidizer_F_kgs = 1.0

        # Initialisation d'un DataFrame pandas pour stocker les données de Outlet
        self.df = pd.DataFrame()

    # Définition de la méthode 'calculate' pour effectuer les calculs thermodynamiques
    def calculate(self):
        # Mise à jour des débits si des valeurs ont été fournies via les ports
        if self.fuel_Inlet.F is not None:
            print('----------------------------------------',self.fuel_Inlet.F)
            self.fuel_F_kgs = self.fuel_Inlet.F

        if self.oxidizer_Inlet.F is not None:
            self.oxidizer_F_kgs = self.oxidizer_Inlet.F

        # Mise à jour du nom du carburant en fonction du fluide entrant
        if self.fuel_Inlet.fluid == "methane":
            self.fuel_name = "CH4"
            M_CH4 = PropsSI('M', 'methane')
            print('-------------------------------------------',self.fuel_F_kgs,M_CH4)
            self.CH4_mols=self.fuel_F_kgs/M_CH4

        # Conversion des noms de carburants en noms reconnus par Cantera
        if self.fuel_name == "C2H6":
            self.fuel_name = "ethane"
        if self.fuel_name == "CH3OH":
            self.fuel_name = "methanol"

        # Mise à jour du nom de l'oxydant en fonction du fluide entrant et calcul des propriétés
        if self.oxidizer_Inlet.fluid == "air":
            self.oxidizer_name = f"O2:2,N2:7.52"
            M_N2 = PropsSI('M', 'Nitrogen')
            M_O2 = PropsSI('M', 'Oxygen')
            fraction_molaire_N2 = 0.79
            fraction_molaire_O2 = 0.21
            # Masse molaire moyenne de l'air sec (N2 et O2 uniquement)
            M_air = fraction_molaire_N2 * M_N2 + fraction_molaire_O2 * M_O2
            print('M_air===================',M_air)
            # Convertir le débit massique total en débit molaire total (en mol/s)
            oxidizer_mols = self.oxidizer_F_kgs / M_air
            print('oxidizer_mols==============' ,oxidizer_mols )
            self.N2_mols = oxidizer_mols *fraction_molaire_N2
            self.O2_mols = oxidizer_mols *fraction_molaire_O2

            # Calcul des débits massiques partiels pour N2 et O2
            N2_F_kgs = self.N2_mols * M_N2
            O2_F_kgs = self.O2_mols * M_O2

        if self.oxidizer_Inlet.fluid == "oxygen":
            self.oxidizer_name = f"O2:1"
            M_O2 = PropsSI('M', 'Oxygen')
            oxidizer_mols = self.oxidizer_F_kgs / M_O2
            self.O2_mols = self.oxidizer_F_kgs/M_O2
            O2_F_kgs = self.oxidizer_F_kgs




        # Calcul de phi en fonction du rapport molaire O2 dans les produits
        if self.products_O2_molRatio is not None:
            # Équilibre à phi=1
            if self.oxidizer_Inlet.fluid == "air": 
                self.phi = 1.0 / (1.0 + 3.76 / self.products_O2_molRatio)
            if self.oxidizer_Inlet.fluid == "oxygen":
                self.phi = 1.0 / (1.0 + 1.0 / self.products_O2_molRatio)

        # Calcul de l'excès d'air en fonction de phi
        if self.AIR_EXCESS is not None:
            self.phi = 1 / ((self.AIR_EXCESS) + 1)
        if self.phi is not None:
            self.AIR_EXCESS = ((1 / self.phi) - 1)
        
        # Affichage de l'excès d'air, de phi et du rapport molaire O2
        print('self.AIR_EXCESS,self.phi,self.products_O2_molRatio',self.AIR_EXCESS,self.phi,self.products_O2_molRatio)


        
        # Appel de la méthode 'Heating_value' pour calculer les valeurs calorifiques
        self.Heating_value()
        # Mise à jour du DataFrame avec les résultats
        self.df = pd.DataFrame({
            'Source': [
                self.Timestamp, self.fuel_name, self.oxidizer_name, self.LHV, self.HHV,
                self.Total_Latent_heat_MJ_kgFuel, self.LHV_kWh_Nm3, self.HHV_kWh_Nm3,
                self.LHV_kWh_Sm3, self.HHV_kWh_Sm3, self.Q_comb_LHV, self.Q_comb_HHV,
                oxidizer_mols,
                self.N2_mols if self.oxidizer_Inlet.fluid == "air" else 0,  # mol/s
                self.O2_mols,  # mol/s
                self.oxidizer_F_kgs,
                N2_F_kgs if self.oxidizer_Inlet.fluid == "air" else 0,  # kg/s
                O2_F_kgs,  # kg/s
            ]
        }, index=[
            'Timestamp', 'fuel_name', 'oxidizer_name', 'comb_LHV (MJ/kg)', 'comb_HHV (MJ/kg)',
            'Total_Latent_heat_MJ_kgFuel', 'LHV_kWh_Nm3', 'HHV_kWh_Nm3',
            'LHV_kWh_Sm3', 'HHV_kWh_Sm3', 'Q_comb_LHV (kW)', 'Q_comb_HHV (kW)','oxidizer (mol/s)',
            'N2_mols (mol/s)', 'O2_mols (mol/s)','oxidizer (kg/s)', 'N2_F_kgs (kg/s)', 'O2_F_kgs (kg/s)'
        ])
        
        self.heat_losses()

    # Définition de la méthode 'Heating_value' pour calculer les valeurs calorifiques
    def Heating_value(self):
        # Création d'un objet eau pour les calculs de propriétés
        water = ct.Water()
        # Réglage de la température et de la qualité de l'eau à 298 K et phase liquide
        water.TQ = 298, 0
        h_liquid = water.h
        # Réglage de la qualité de l'eau à phase gazeuse
        water.TQ = 298, 1
        h_gas = water.h

        # Réglage des conditions initiales du gaz avant combustion
        self.gas.TP = 298, ct.one_atm
        self.gas.set_equivalence_ratio(phi=1, fuel=self.fuel_name, oxidizer=self.oxidizer_name)

        h1 = self.gas.enthalpy_mass
        print('h1================================================',h1)
        Y_fuel = self.gas[self.fuel_name].Y[0]

        # Simulation de la combustion
        X_products = {
            "CO2": self.gas.elemental_mole_fraction("C"),
            "H2O": 0.5 * self.gas.elemental_mole_fraction("H"),
            "N2": 0.5 * self.gas.elemental_mole_fraction("N"),
        }

        self.gas.TPX = None, None, X_products
        Y_H2O = self.gas["H2O"].Y[0]
        h2 = self.gas.enthalpy_mass

        # Calcul de la valeur calorifique inférieure (LHV) et supérieure (HHV)
        self.LHV = -(h2 - h1) / Y_fuel / 1e6  # LHV en MJ/kg
        self.HHV = -(h2 - h1 + (h_liquid - h_gas) * Y_H2O) / Y_fuel / 1e6  # HHV en MJ/kg
        self.Total_Latent_heat_MJ_kgFuel = self.HHV - self.LHV

        # Calcul de la densité à des températures normales et standard
        rho_N = PropsSI("D", "T", 0 + 273.15, "P", 101325, self.fuel_name)  # Densité à 0°C en kg/m^3
        rho_S = PropsSI("D", "T", 20 + 273.15, "P", 101325, self.fuel_name)  # Densité à 20°C en kg/m^3

        # Conversion des valeurs calorifiques en kWh/m^3
        self.LHV_kWh_Nm3 = self.LHV * rho_N / 3.6
        self.HHV_kWh_Nm3 = self.HHV * rho_N / 3.6
        self.LHV_kWh_Sm3 = self.LHV * rho_S / 3.6
        self.HHV_kWh_Sm3 = self.HHV * rho_S / 3.6

        # Calcul de la quantité de chaleur libérée par la combustion
        self.Q_comb_HHV = self.HHV * self.fuel_F_kgs * 1e3  # HHV en kW
        self.Q_comb_LHV = self.LHV * self.fuel_F_kgs * 1e3  # LHV en kW

    def heat_losses(self):
        # Ici, nous définissons la nouvelle composition de carburant et d'oxydant
        fuel_composition = f"CH4:{round(self.CH4_mols,2)}"
        if self.oxidizer_Inlet.fluid == "air":
            oxidizer_composition = f"O2:{round(self.O2_mols,2)},N2:{round(self.N2_mols,2)}"
        else:
            oxidizer_composition = f"O2:{round(self.O2_mols,2)}"
        
        print(f"Fuel composition: {fuel_composition}")
        print(f"Oxidizer composition: {oxidizer_composition}")
        
        # Créer une instance de Solution
        gas = ct.Solution('gri30.yaml')
        # Définir les fractions molaires directement pour créer le mélange de gaz
        gas.X = f"{fuel_composition},{oxidizer_composition}"
   
        # Équilibre le gaz à une température et une pression données
        gas.TP = 300.0, ct.one_atm
        gas.equilibrate('HP')
        # Enregistrez l'enthalpie à l'état équilibré
        h_initial = gas.enthalpy_mass

        # Calculer phi après équilibrage en utilisant les nouvelles fractions molaires
        self.phi = gas.equivalence_ratio()
        
        print(f"State after equilibration: T = {gas.T:.2f} K, P = {gas.P:.2f} Pa, rho = {gas.density:.2f} kg/m³")
        print(f"Phi recalculé après équilibrage = {self.phi:.3f}")
        print(gas())

         # État final à 110°C (383 K)
        gas.TP = 25+273.15, ct.one_atm
        # Pas besoin d'équilibrer de nouveau, nous voulons juste l'enthalpie à cette température
        h_final = gas.enthalpy_mass

        self.mass_flow_rate=self.fuel_F_kgs+self.oxidizer_F_kgs
        # La chaleur perdue sera la différence d'enthalpie par unité de masse multipliée par la masse totale du gaz
        heat_lost = (h_initial - h_final) * self.mass_flow_rate  # masse_flow_rate doit être définie ailleurs dans la classe

        print(f"Chaleur perdue (jusqu'à 210°C): {heat_lost/1000:.2f} kW")


# utilisation du module
from ThermodynamicCycles.Combustion import Combustor_cantera
from ThermodynamicCycles.Source import Source
from ThermodynamicCycles.Connect import Fluid_connect

fioul_SOURCE = Source.Object()
oxidizer_SOURCE = Source.Object()
COMB = Combustor_cantera.Object()

fioul_SOURCE.F = 1
fioul_SOURCE.fluid = "methane"
fioul_SOURCE.Ti_degC = -5.0
fioul_SOURCE.Pi_bar = 1.01325 + 0.030  # 30 millibar

oxidizer_SOURCE.fluid = "air" #"oxygen"
oxidizer_SOURCE.F = 17.2
oxidizer_SOURCE.Ti_degC = 15.0
oxidizer_SOURCE.Pi_bar = 1.01325 + 0.2  # 30 millibar

#COMB.phi = 1.0
#COMB.AIR_EXCESS = 0.03*0

fioul_SOURCE.calculate()
oxidizer_SOURCE.calculate()

Fluid_connect(COMB.fuel_Inlet, fioul_SOURCE.Outlet)
Fluid_connect(COMB.oxidizer_Inlet, oxidizer_SOURCE.Outlet)
COMB.calculate()

print(fioul_SOURCE.df)
print(oxidizer_SOURCE.df)
print(COMB.df)



from ThermodynamicCycles.FluidPort.FluidPort import FluidPort
from CoolProp.CoolProp import PropsSI
import pandas as pd 
from datetime import datetime
from scipy.optimize import fsolve

import math

class Object:
    def __init__(self):
        self.Timestamp=datetime.now().strftime("%Y-%m-%d %H:%M:%S")
   
        self.Air_Inlet = FluidPort(fluid='air', F=15/3.6,P=101325)
        self.Air_Outlet = FluidPort(fluid='air',F=15/3.6,P=101325)
        self.Fluid_Inlet = FluidPort(fluid='water',P=101325)
        self.Fluid_Outlet = FluidPort(fluid='water',P=101325)

#########paramètre de design
        self.fluid="water" # "hydrocarbure léger" , "water" 
        self.Ti_fluid=165 #	Température d'entrée du fluide de procédé
        self.To_fluid=None	#Température de sortie Fluide de Procédé
        self.Cp_fluid=None	#Capacité thermique massique J/ Kg K
        self.F_fluid=None	#débit du fluide procédé     
        self.Ti_air=25	#température d'entrée de l'air ambiant
        self.rho_air=None #1.201	
        self.Cp_air=None
        self.TminAmb=5 #	température minimale ambiante
        
        self.largeur_baie=6 # Largeur (m)	3,5
        self.L_tube=10
        self.L_tube_max=18 # Tube	Longueur max (m)	18

        self.U=None
        self.DTLM=None
        self.UA=None

###################géométrie de référence
        self.L_tube=3 
        self.diametre_ext_tube=25.4 # Diamètre extérieur (mm)	25,4
        self.rapport_ailetage=20.5 # Rapport d'ailetage (m2 aileté/m2 nu)	20,5
        self.pas_triangulaire=63.5#     Baie	pas triangulaire (mm)	63,5
 
        self.nb_faisceaux=2# Nombre de faisceau	2

        self.rendement_statique_ventilateur=0.6# Rendement statique	0,6
        self.Pression_statique_ventilateur=150# Pression statique	150
        self.rendement_transmission_ventilateur=0.95# Rendement de trasmission	0,95

        

        #résult
        self.Qth_fluid=None
        self.nb_rangs=None
        self.Ntr=None #nombre de tube par rang
        self.Ntf=None # nombre de tube par faisceau
        self.Sf=None #Surface faciale Sf m2
        self.Stn=None #Surface de tube Nus
        self.surface_ailetee=None # Surface Ailetée
        self.V_air=None #m/s
        self.Qth_air=None

        self.nb_baie_design=None
        self.S_sol_design=None
        self.Stn_design=None
        self.surface_ailetee_design=None
        self.dmin_vent=None
        self.d_vent=None

        #Nombres Adim
        self.R1=None
        self.R2=None
        self.R3=None

        #Détermination de la ventilation
        self.nb_vent_baie=2
        self.S_vent=None

        self.F_vent_m3h=None
        self.F_air_m3h=None


                # Détermination du nombre de rangs 

        if (self.Ti_fluid-self.Ti_air)<=10:
           self.nb_rangs=3
        if (self.Ti_fluid-self.Ti_air)<=50 and (self.Ti_fluid-self.Ti_air)>10:
            self.nb_rangs=4
        if (self.Ti_fluid-self.Ti_air)<=90 and (self.Ti_fluid-self.Ti_air)>50:
            self.nb_rangs=6
        if (self.Ti_fluid-self.Ti_air)<=140 and (self.Ti_fluid-self.Ti_air)>90:
            self.nb_rangs=7
        else:
            self.nb_rangs=3

        self.df = pd.DataFrame()
        
    def calculate(self):

        self.Ti_fluid=self.Fluid_Inlet.T-273.15
        self.Cp_fluid=self.Fluid_Inlet.cp
        self.F_fluid=self.Fluid_Inlet.F
        self.fluid=self.Fluid_Inlet.fluid

        self.Ti_air=self.Air_Inlet.T-273.15
        self.Cp_air=self.Air_Inlet.cp
        self.rho_air=self.Air_Inlet.rho
 

        #Détermination de la quantité de chaleur à évacuer :
        self.Qth_fluid=self.F_fluid*self.Cp_fluid*(self.Ti_fluid-self.To_fluid)

        #Détermination du coefficient de transmission thermique globale
        if self.U is None:
            if self.fluid=="water":
                self.U=850
            if self.fluid=="hydrocarbure léger":
                self.U=540
            if self.fluid=="Gasoil léger":
                self.U=400


  

        # Détermination de la géométrie de la baie

        # le nombre de tubes par rang
        self.Ntr=int((self.largeur_baie)/2/(self.pas_triangulaire*10**(-3)))

        self.Ntf=self.nb_rangs*self.Ntr

        if self.L_tube>=self.L_tube_max:
            self.L_tube=self.L_tube_max

        self.Sf=self.largeur_baie*self.L_tube

        self.Stn=round(self.Ntf*math.pi*self.diametre_ext_tube*10**(-3)*self.L_tube*self.nb_faisceaux,2)
        self.surface_ailetee=self.Stn*self.rapport_ailetage

        if self.nb_rangs==4:
            self.V_air=3.55
        if self.nb_rangs==5:
            self.V_air=3.1
        if self.nb_rangs==6:
            self.V_air=2.75
        if self.nb_rangs==7:
            self.V_air=2.5

        self.R3=(self.Ti_fluid-self.To_fluid)/(self.Ti_fluid-self.Ti_air)
        self.R1=(self.U*self.Stn)/(self.V_air*self.Sf*self.rho_air*self.Cp_air)
        # Utilisation de fsolve pour trouver R2
        equation = lambda R2: self.R1 - math.log((1 - R2) / (1 - self.R3)) / ((self.R3 / R2) - 1)
        R2 = fsolve(equation, x0=0.1)[0]  # x0 est la valeur initiale, ajustez selon vos besoins
        self.R2 = R2  # Mettez à jour self.R2 avec la valeur calculée

        #calcul de self.To_air
        self.To_air=round(self.Ti_air+self.R2*(self.Ti_fluid-self.Ti_air),1)
        self.DTLM=((self.Ti_fluid-self.To_air)-(self.To_fluid-self.Ti_air))/math.log((self.Ti_fluid-self.To_air)/(self.To_fluid-self.Ti_air))
        


        self.Qth_air=self.V_air*self.Sf*self.rho_air*self.Cp_air*(self.To_air-self.Ti_air)

        self.nb_baie=self.Qth_fluid/self.Qth_air
        self.nb_baie_design=round(self.Qth_fluid/self.Qth_air,0)
        self.S_sol_design=self.nb_baie_design*self.largeur_baie*self.L_tube
        self.Stn_design=self.nb_baie_design*self.Stn
        self.surface_ailetee_design=self.nb_baie_design*self.surface_ailetee
        self.UA=self.Qth_fluid/self.DTLM
        self.Qth=self.UA*self.DTLM
 




        #design 
        
        self.dmin_vent=(4*0.4*self.Sf/math.pi/self.nb_vent_baie)**(0.5)
        if self.d_vent<=self.dmin_vent:
            self.d_vent=self.dmin_vent

  
        self.F_vent_m3s=round(self.V_air*self.Sf*(273.15+self.Ti_air)/((273.15+self.TminAmb)*self.nb_vent_baie),1)
        self.F_vent_m3h=self.F_vent_m3s*3600


        self.P_vent=self.F_vent_m3h/3600*self.Pression_statique_ventilateur/(self.rendement_statique_ventilateur*self.rendement_transmission_ventilateur)*(273.15+self.Ti_air)/(273.15+self.TminAmb)
        self.P_vent = math.ceil(self.P_vent / 5000) * 5000 #arrondir la puissance du ventillateur à installer

        self.nb_vent=self.nb_vent_baie*self.nb_baie_design
        self.F_air_m3h=self.nb_vent*self.F_vent_m3h
        self.nb_tube=self.Ntf*self.nb_vent
        self.P_elec=self.P_vent*self.nb_vent
                                  

        self.Fluid_Outlet.T=self.To_fluid+273.15
        self.Fluid_Outlet.F=self.Fluid_Inlet.F
        self.Fluid_Outlet.P=self.Fluid_Inlet.P
        self.Fluid_Outlet.calculate_properties()

        self.Air_Outlet.T=self.To_air+273.15
        self.Air_Outlet.F=self.Air_Inlet.F
        self.Air_Outlet.P=self.Air_Inlet.P-self.Pression_statique_ventilateur
        self.Air_Outlet.calculate_properties()

        self.df = pd.DataFrame({'DTLM_HEX': [self.Timestamp,self.Qth_fluid/1000,self.U,self.nb_rangs,self.Ntr,self.Ntf,self.Sf,
                                             self.Stn,self.surface_ailetee,self.V_air,self.To_air,self.Qth_air/1000,self.nb_baie,self.nb_baie_design,self.S_sol_design,self.Stn_design,self.surface_ailetee_design,self.dmin_vent,self.d_vent,self.F_vent_m3h,self.F_vent_m3s,
                                             self.P_vent/1000,self.F_air_m3h,self.nb_tube,self.P_elec/1000,self.nb_vent
                                             ], },
                      index = ['Timestamp','quantité de chaleur à évacuer : Qth_fluid (kW)','coefficient de transmission thermique globale :U ((W/m2-K))','self.nb_rangs','nb tubes par rangée','nb tubes par faisceaux','Surface faciale Sf',
                               'Surface des tubes nus Stn','self.surface_ailetee','self.V_air','self.To_air','Qth_air(kW)','self.nb_baie','self.nb_baie_design',"Surface au sol de l'équipement","Surface d'échange nue (m2)","Surface Aileté totale","diamètre min d'un ventilateur","diamètre d'un ventilateur retenu","self.F_vent_m3h","self.F_vent_m3s","P_vent (kW)","self.F_air_m3h","self.nb_tube",'P_elec (kW)','self.nb_vent'])     





from setuptools import setup, find_packages, find_namespace_packages


# Add the documentation URL to the long description
with open("README.md","r") as fh:
    long_description=fh.read()



setup(
    name='energysystemmodels',


    version='20260407001',

    description='Energy systems models are the mathematical models that are developed in order to represent as reliably as possible various energy-related problems.',

    
    classifiers =[   'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
    'Programming Language :: Python :: 3.10', "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",],
    package_dir = {"":"src"},
    packages=find_packages(where="src"),
    package_data={"": ["*.ini", "*.json"], },
   
    #py_modules=['AHU.HeatingCoil'],
    #packages=['ahu', 'ahu.Coil'],
    python_requires=">=3.7",
    

    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=["pandas","pyfluids","pylab-sdk~=1.3.2","matplotlib","tkintertable~=1.3.3","numpy","pyqtgraph","thermochem","sip", "PyQt-builder","PyQt5","scikit-learn","beautifulsoup4","tqdm","statsmodels","python-docx","pvlib","gekko","networkx",],
    extra_require={"dev":["pytest>=3.7",],},
    #url="https://github.com/ZoheirHADID/EnergySystemModels",
    author="Zoheir HADID",
    author_email="zoheir.hadid@gmail.com",
    project_urls={
        'Documentation': 'https://energysystemmodels-tutorial.readthedocs.io/en/latest/',
    },
    )


class Object:
    def __init__(self, orientation, Tp, Ta, W=None, L=None, H=None, Cp=1007, beta=0.0034, k=0.0261, mu=0.0000185, rhoref=1.201, e=0.85):
        self.orientation = orientation
        self.Tp = Tp
        self.Ta = Ta
        self.W = W
        self.L = L
        self.H = H
        self.Cp = Cp
        self.beta = beta
        self.k = k
        self.mu = mu
        self.rhoref = rhoref
        self.e = e
        self.g = 9.81  # Acceleration due to gravity (m/s^2)
        self.sigma = 5.67e-8  # Stefan-Boltzmann constant (W/m^2-K^4)

    def calculate(self):
        if self.orientation == 'horizontal_down':
            return self._horizontal_plate_facing_down_heat_transfer()
        elif self.orientation == 'horizontal_up':
            return self._horizontal_plate_facing_up_heat_transfer()
        elif self.orientation == 'vertical':
            return self._vertical_plate_heat_transfer()
        else:
            raise ValueError("Invalid orientation. Choose from 'horizontal_down', 'horizontal_up', or 'vertical'.")

    def _horizontal_plate_facing_down_heat_transfer(self):
        # Calculated Parameters
        Tf = (self.Tp + self.Ta) / 2  # Film temperature (°C)
        v = self.mu / self.rhoref  # Kinematic viscosity (m^2/s) at reference density
        rho = self.rhoref * (1 - self.beta * (Tf - 20))  # Density at film temperature (kg/m^3)
        a = self.k / (rho * self.Cp)  # Thermal diffusivity (m^2/s)

        # Dimensionless Numbers
        Pr = v / a  # Prandtl number
        Gr = self.g * self.beta * (self.Tp - self.Ta) * ((self.W * self.L) / (2 * self.W + 2 * self.L))**3 / v**2  # Grashof number
        Ra = Gr * Pr  # Rayleigh number

        # Nusselt Number for horizontal plate facing down
        Nu = 0.27 * Ra**0.25 if Ra > 1e4 and Ra < 1e7 else (0.54 * Ra**0.25 if Ra >= 1e7 else 1)

        # Heat Transfer Coefficient
        h = Nu * self.k / ((self.W * self.L) / (2 * self.W + 2 * self.L))

        # Convective Heat Transfer
        q_conv = h * self.W * self.L * (self.Tp - self.Ta)

        # Radiative Heat Transfer
        q_rad = self.sigma * self.W * self.L * self.e * ((self.Tp + 273.15)**4 - (self.Ta + 273.15)**4)

        # Total Heat Transfer
        q_total = q_conv + q_rad

        return q_total

    def _horizontal_plate_facing_up_heat_transfer(self):
        # Film temperature
        Tf = (self.Tp + self.Ta) / 2.0

        # Fluid properties at film temperature
        Pr = self.mu / (self.k / (self.Cp * self.rhoref))  # Prandtl number
        rho = self.rhoref * (1 - self.beta * (Tf - 20))  # Density at Tf

        # Grashof number calculation
        Gr = self.g * self.beta * (self.Tp - self.Ta) * ((self.W * self.L) / (2 * self.W + 2 * self.L))**3 / self.mu**2

        # Rayleigh number
        Ra = Gr * Pr

        # Nusselt number calculation based on Ra
        Nu = 0.15 * (Ra**0.33)

        # Average heat transfer coefficient
        h = Nu * self.k / ((self.W * self.L) / (2 * self.W + 2 * self.L))

        # Convective heat transfer
        q_conv = h * self.W * self.L * (self.Tp - self.Ta)

        # Radiative heat transfer
        q_rad = self.sigma * self.W * self.L * self.e * ((self.Tp + 273.15)**4 - (self.Ta + 273.15)**4)

        # Total heat transfer
        q_total = q_conv + q_rad

        return q_total

    def _vertical_plate_heat_transfer(self):
        if self.H is None:
            raise ValueError("Height (H) must be provided for vertical plate heat transfer calculation.")

        # Film temperature
        Tf = (self.Tp + self.Ta) / 2
        # Fluid properties at film temperature
        v = self.mu / self.rhoref  # Kinematic viscosity
        alpha = self.k / (self.rhoref * self.Cp)  # Thermal diffusivity
        Pr = v / alpha  # Prandtl number

        # Density at film temperature
        rho = self.rhoref * (1 - self.beta * (Tf - 20))

        # Grashof number calculation
        Gr = self.g * self.beta * (self.Tp - self.Ta) * self.H**3 / v**2

        # Rayleigh number
        Ra = Gr * Pr

        # Nusselt number calculation
        if Ra < 1e9:
            Nu = (0.68 + (0.67 * Ra ** (1 / 4)) / (1 + (0.492 / Pr) ** (9 / 16)) ** (4 / 9)) ** 2
        else:
            Nu = (0.825 + (0.387 * Ra ** (1 / 6)) / ((1 + (0.492 / Pr) ** (9 / 16)) ** (8 / 27))) ** 2

        # Average heat transfer coefficient
        h = Nu * self.k / self.H

        # Convective heat transfer
        q_conv = h * self.W * self.H * (self.Tp - self.Ta)

        # Radiative heat transfer
        q_rad = self.sigma * self.W * self.H * self.e * ((self.Tp + 273.15)**4 - (self.Ta + 273.15)**4)

        # Total heat transfer
        q_total = q_conv + q_rad

        return q_total


from datetime import date
import json
import os
from dateutil.parser import parse
import pandas as pd
import matplotlib.pyplot as plt
from matplotlib.patches import Rectangle
import numpy as np  # nécessaire pour cos, sin, deg2rad

current_directory = os.path.dirname(os.path.abspath(__file__))
coefficients_file_path = os.path.join(current_directory, 'coefficients_elec.json')
#print("coefficients_file_path =",coefficients_file_path )




class input_Facture:
    def __init__(self, start,end,depassement_PS_pointe=0, depassement_PS_HPH=0, depassement_PS_HCH=0, depassement_PS_HPB=0, depassement_PS_HCB=0, heures_depassement=0,kWh_pointe=0,kWh_HPH=0,
        kWh_HCH=0,    kWh_HPB=0,        kWh_HCB=0):

        # Si start n'est pas déjà un objet date, le convertir en date
        if not isinstance(start, date):
            start = parse(start).date()

        # Si end n'est pas déjà un objet date, le convertir en date
        if not isinstance(end, date):
            end = parse(end).date()

        self.start = start
        self.end = end


        self.depassement_PS_pointe = depassement_PS_pointe
        self.depassement_PS_HPH = depassement_PS_HPH
        self.depassement_PS_HCH = depassement_PS_HCH
        self.depassement_PS_HPB = depassement_PS_HPB
        self.depassement_PS_HCB = depassement_PS_HCB
        self.heures_depassement = heures_depassement
        self.kWh_pointe=kWh_pointe
        self.kWh_HPH=kWh_HPH
        self.kWh_HCH=kWh_HCH
        self.kWh_HPB=kWh_HPB
        self.kWh_HCB=kWh_HCB

class input_Tarif:
    def __init__(self, 
                 c_euro_kWh_pointe=0.0, 
                 c_euro_kWh_HPB=0.0, 
                 c_euro_kWh_HCB=0.0, 
                 c_euro_kWh_HPH=0.0, 
                 c_euro_kWh_HCH=0.0, 
                 c_euro_kwh_CSPE_TICFE=None, 
                 c_euro_kWh_certif_capacite_pointe=0.0, 
                 c_euro_kWh_certif_capacite_HPH=0.0, 
                 c_euro_kWh_certif_capacite_HCH=0.0, 
                 c_euro_kWh_certif_capacite_HPB=0.0, 
                 c_euro_kWh_certif_capacite_HCB=0.0, 
                 c_euro_kWh_ENR=0.0, 
                 c_euro_kWh_ARENH=0.0):
        self.c_euro_kWh_pointe = c_euro_kWh_pointe
        self.c_euro_kWh_HPB = c_euro_kWh_HPB
        self.c_euro_kWh_HCB = c_euro_kWh_HCB
        self.c_euro_kWh_HPH = c_euro_kWh_HPH
        self.c_euro_kWh_HCH = c_euro_kWh_HCH
        self.c_euro_kwh_CSPE_TICFE = c_euro_kwh_CSPE_TICFE
        self.c_euro_kWh_certif_capacite_pointe=c_euro_kWh_certif_capacite_pointe
        self.c_euro_kWh_certif_capacite_HPH=c_euro_kWh_certif_capacite_HPH
        self.c_euro_kWh_certif_capacite_HCH=c_euro_kWh_certif_capacite_HCH
        self.c_euro_kWh_certif_capacite_HPB=c_euro_kWh_certif_capacite_HPB
        self.c_euro_kWh_certif_capacite_HCB=c_euro_kWh_certif_capacite_HCB
        self.c_euro_kWh_ENR = c_euro_kWh_ENR
        self.c_euro_kWh_ARENH = c_euro_kWh_ARENH


class input_Contrat:
    def __init__(self, depassement_PS_pointe=None, domaine_tension=None, heures_depassement=None, PS_pointe=None, PS_HPH=None, PS_HCH=None, PS_HPB=None, PS_HCB=None, version_utilisation=None,pourcentage_ENR=0,abonnement_annuel=0,cadre_contractuel="contrat unique",nb_cellules_secours=0,km_secours_aerien=0,km_secours_souterrain=0):
        self.domaine_tension = domaine_tension
        self.PS_pointe = PS_pointe
        self.PS_HPH = PS_HPH
        self.PS_HCH = PS_HCH
        self.PS_HPB = PS_HPB
        self.PS_HCB = PS_HCB
        self.version_utilisation = version_utilisation
        self.pourcentage_ENR=pourcentage_ENR
        self.abonnement_annuel = abonnement_annuel
        self.cadre_contractuel = cadre_contractuel
        self.nb_cellules_secours=nb_cellules_secours
        self.km_secours_aerien=km_secours_aerien
        self.km_secours_souterrain=km_secours_souterrain

class TurpeCalculator:
    def __init__(self,contrat,tarif,facture, coefficients_file="coefficients.json"):
        self.contrat = contrat
        self.tarif=tarif
        self.facture=facture
       
        self.coefficients_file = os.path.join(current_directory, coefficients_file)
       

        self.coefficients = None
        self.ajustables = []
        self.euro_CS_fixe=None

        self.euro_an_CACS=0.0 # intégrer après dans le calcul
        self.euro_an_CR=0.0# intégrer après dans le calcul
        self.euro_an_CER=0.0 # intégrer après dans le calcul
        self.euro_an_CI=0.0 # intégrer après dans le calcul
        self.euro_CTA=0.0
        self.euro_an_CTA=0.0
        self.euro_CSPE_TICFE=0.0
        self.euro_abonnement= 0.0
      



    def get_TURPE_coef(self, facture, version_utilisation, domaine_tension,cadre_contractuel="contrat unique"):
        with open(coefficients_file_path, 'r') as fichier:
            donnees = json.load(fichier)

        for coefficient in donnees["coefficients"]:
            start_date = parse(coefficient["start_date"]).date()
            expiration_date = parse(coefficient["expiration_date"]).date()
            if (start_date <= facture.start <= expiration_date and
                start_date <= facture.end <= expiration_date and
                coefficient["version_utilisation"] == version_utilisation and
                coefficient["domaine_tension"] == domaine_tension and
                coefficient.get("cadre_contractuel", cadre_contractuel) == cadre_contractuel):
                #print("coefficient = ",coefficient)
                return coefficient

        return None



# Example usage:
    def calculate_taxes_contrib(self,coeff):
        #coeff = self.get_TURPE_coef(self.facture, self.contrat.version_utilisation, self.contrat.domaine_tension)
        self.calculate_euro_CTA(coeff)
        self.calculate_euro_CSPE_TICFE(coeff)
        self.euro_taxes_contrib=round(self.euro_CTA+self.euro_CSPE_TICFE,2)
        #print("euro_taxes_contrib = ",self.euro_taxes_contrib)

    def calculate_turpe(self):
        self.df=None
        self.euro_an_CS=None
        self.euro_an_TURPE_CTA=None
        coeff = self.get_TURPE_coef(self.facture, self.contrat.version_utilisation, self.contrat.domaine_tension,self.contrat.cadre_contractuel)
        self.kWh_Total = self.facture.kWh_HPH +self.facture.kWh_HCH + self.facture.kWh_HPB + self.facture.kWh_HCB + self.facture.kWh_pointe
        #print("kWh_Total = " ,self.kWh_Total )

        if self.contrat.domaine_tension == "HTA":
            self.calculate_euro_an_CS_fixe(coeff, self.contrat)
            self.calculate_euro_CS_variable(coeff, self.facture)
      
     
            self.euro_mois_CMDPS = self.calculate_euro_mois_CMDPS(coeff, self.facture)

        elif self.contrat.domaine_tension == "BT > 36 kVA":
            self.calculate_euro_an_CS_fixe(coeff, self.contrat)
            self.calculate_euro_CS_variable(coeff, self.facture)
            self.euro_mois_CMDPS = self.calculate_euro_mois_CMDPS(coeff, self.facture)

        elif self.contrat.domaine_tension == "BT < 36 kVA":
            self.calculate_euro_an_CS_fixe(coeff, self.contrat)
            self.calculate_euro_CS_variable(coeff, self.facture)
            self.euro_mois_CMDPS = 0.0
        
        # self.euro_CS_fixe = self.calculate_euro_CS_fixe(self.euro_an_CS_fixe, self.facture)
        
        self.calculate_nb_jour()  # Appel de la méthode pour calculer le nombre de jours
        self.calculate_euro_CS_fixe()
        self.calculate_euro_CC(coeff)
        self.calculate_euro_CG(coeff)
        self.calculate_euro_CACS(coeff)
        self.calculate_taxes_contrib(coeff)
        self.calculate_euro_TURPE()
        self.calculate_euro_an_TURPE()
        # self.calculate_euro_total()
        self.calculate_montant(coeff)
        


        self.euro_an_CS=self.euro_an_CS_fixe + self.euro_CS_variable
        self.euro_CS= self.euro_CS_fixe + self.euro_CS_variable
        self.euro_an_TURPE_CTA=self.euro_an_TURPE + self.euro_an_CTA

        df_data_cout = [
            ("Nombre de jours", self.nb_jour),
            ("Fourniture (€)", self.euro_fourniture),
            ("Distribution (TURPE) (€)", self.euro_TURPE),
            ("Taxes et contributions (€)", self.euro_taxes_contrib),
            ("Facture HTVA (€)", self.euro_total),
            ("Facture TTC (€)", self.euro_total*1.2),
            ("MWh Total", self.kWh_Total / 1000.0),
            ("cout du MWh (€HTVA/MWh)", round(self.euro_total / (self.kWh_Total / 1000.0), 2)),
            ("cout du MWh (€Hors taxes et contributions /MWh)", round((self.euro_total - self.euro_taxes_contrib) / (self.kWh_Total / 1000.0), 2)),
            ("cout du MWh (€ fourniture /MWh)", round(self.euro_fourniture / (self.kWh_Total / 1000.0), 2)),
            ("cout du MWh (€ distribution /MWh)", round((self.euro_total - self.euro_fourniture - self.euro_taxes_contrib) / (self.kWh_Total / 1000.0), 2))
        ]

        df_index_cout = [
            "self.nb_jour",
            "self.euro_fourniture",
            "self.euro_TURPE",
            "self.euro_taxes_contrib",
           
            "self.euro_total",
            "self.euro_total * 1.2",
            "self.kWh_Total / 1000.0",
            "cout du MWh (€HTVA/MWh)",
            "cout du MWh (€Hors taxes et contributions /MWh)",
            "cout du MWh (€ fourniture /MWh)",
            "cout du MWh (€ distribution /MWh)"


        ]

        self.df_couts = pd.DataFrame(df_data_cout, index=df_index_cout, columns=["Nom de variable", "Valeur"])
        df_data_fourniture = [
            ("Heures pleines été",self.euro_HPB),
            ("Heures creuses été",self.euro_HCB),
            ("Heure pleines hiver",self.euro_HPH),
            ("heure creuse hiver",self.euro_HCH),
            ("heures pointe",self.euro_pointe),
            ("self.euro_capacite", self.euro_capacite),
            ("self.euro_abonnement", self.euro_abonnement),
            ("Accès Régulé à l'Électricité Nucléaire Historique-ARENH", self.euro_ARENH),
            ("Distribution (sans ARENH)", self.euro_fourniture),
            ("Distribusiton (avec ARENH)", self.euro_fourniture)]
        df_index_fourniture = [
            "self.euro_HPB",
            "self.euro_HCB",
            "self.euro_HPH",
            "self.euro_HCH",
            "self.euro_pointe",
            "self.euro_capacite",
            "self.euro_abonnement",
            "self.euro_ARENH",
            "self.euro_fourniture (sans ARENH)",
            "self.euro_fourniture (avec ARENH)"
        ]
        self.df_fourniture = pd.DataFrame(df_data_fourniture, index=df_index_fourniture, columns=["Nom de variable", "Valeur"])
        df_data_distribution = [
            ("Composante de gestion - CG (€)", self.euro_CG),
            ("Composante de comptage - CC (€)", self.euro_CC),
            ("Part Puissance - CS Fixe (€)", self.euro_CS_fixe),
            ("Part Énergie - CS Variable (€)", self.euro_CS_variable),
            ("Dépassement puissance souscrite - CMDPS (€)", self.euro_mois_CMDPS),
            ("Alimentation de secours - CACS (€)", self.euro_CACS),
            ("Distribution : TURPE (€)", self.euro_TURPE)]
        df_index_distribution = [
            "self.euro_CG",
            "self.euro_CC",
            "self.euro_CS_fixe",
            "self.euro_CS_variable",
            "self.euro_mois_CMDPS",
            "self.euro_CACS",
            "self.euro_TURPE"
        ]
        self.df_distribution = pd.DataFrame(df_data_distribution, index=df_index_distribution, columns=["Nom de variable", "Valeur"])
        df_data_taxes_contrib = [
            ("CTA (€)", self.euro_CTA),
            ("TICFE/CSPE (€)", self.euro_CSPE_TICFE),
            ("Taxes et contributions (€)", self.euro_taxes_contrib)]
        df_index_taxes_contrib = [
            "self.euro_CTA",
            "self.euro_CSPE_TICFE",
            "self.euro_taxes_contrib"
        ]
        self.df_taxes_contributions = pd.DataFrame(df_data_taxes_contrib, index=df_index_taxes_contrib, columns=["Nom de variable", "Valeur"])

        df_data_annuel = [
            ("Composante de gestion - CG (€/an)", self.euro_an_CG),
            ("Composante de comptage - CC (€/an)", self.euro_an_CC),
            ("Composante de soutirage - fixe + variable (€/an)", self.euro_an_CS),
            ("Part Puissance - CS Fixe (€/an)", self.euro_an_CS_fixe),
            ("Part Énergie - CS Variable (€/an)", self.euro_CS_variable),
            ("Alimentation de secours - CACS (€/an)", self.euro_an_CACS),
            ("Distribustion : TURPE (€/an)", self.euro_an_TURPE),
            ("CTA annuel (€/an)", self.euro_an_CTA),
            ("Distribustion + CTA  (€/an)", self.euro_an_TURPE_CTA),
            ("TICFE/CSPE annuel (€/an)", self.euro_an_CSPE_TICFE)]
        df_index_annuel = [
            "self.euro_an_CG",
            "self.euro_an_CC",
            "self.euro_an_CS",
            "self.euro_an_CS_fixe",
            "self.euro_CS_variable",
            "self.euro_an_CACS",
            "self.euro_an_TURPE",
            "self.euro_an_CTA",
            "self.euro_an_TURPE_CTA",
            "self.euro_an_CSPE_TICFE"
        ]
        self.df_annuel = pd.DataFrame(df_data_annuel, index=df_index_annuel, columns=["Nom de variable", "Valeur"])
        
 
       
        



    def plot(self, title="Facture d'Électricité en € HTVA", figsize=(8, 6)):
        # Vérification des données nécessaires
        if self.euro_fourniture is None or self.euro_TURPE is None or self.euro_taxes_contrib is None:
            print("Certaines composantes nécessaires au graphique ne sont pas définies.")
            return

        labels = ['Fourniture', 'Distribution (TURPE)', 'Taxes et Contributions']
        values = [self.euro_fourniture, self.euro_TURPE, self.euro_taxes_contrib]
        colors = ['#4CAF50', '#2196F3', '#FFC107']  # Vert, Bleu, Jaune

        total = sum(values)

        fig, ax = plt.subplots(figsize=figsize)

        wedges, texts, autotexts = ax.pie(
            values,
            labels=None,  # Pas de labels autour du cercle
            autopct='%1.1f%%',
            startangle=90,
            colors=colors,
            wedgeprops=dict(width=0.4, edgecolor='w'),
            pctdistance=0.8,
            textprops={'fontsize': 12}
        )

        ax.axis('equal')  # Cercle parfait
        plt.title(title, fontsize=12, pad=16)

        # Ajout du total au centre du donut
        ax.text(0, 0, f"{total:,.2f} €", ha='center', va='center', fontsize=14, fontweight='bold')

        # Positionnement de la légende en bas à gauche du donut
        y_start = -0.05  # plus bas dans l'axe (peut être négatif)
        y_step = 0.07    # plus petit pour réduire l'espacement
        x_rect = -0.12   # carré un peu plus à gauche
        x_text = -0.05   # texte un peu plus à gauche aussi

        for i, (label, val, color) in enumerate(zip(labels, values, colors)):
            y = y_start - i * y_step
            rect = Rectangle((x_rect, y - 0.02), 0.04, 0.04, facecolor=color, transform=ax.transAxes, clip_on=False)
            ax.add_patch(rect)
            ax.text(x_text, y, f"{label} : {val:,.2f} €",
                    transform=ax.transAxes,
                    ha='left', va='center',
                    fontsize=11, weight='bold')

        plt.tight_layout()
        plt.show()



    def plot_detail(self, title="Détail des Composantes de la Facture d'Électricité", figsize=(16, 6)):
        # Vérification des données nécessaires
        necessary_attrs = [
            'euro_HPB', 'euro_HCB', 'euro_HPH', 'euro_HCH', 'euro_pointe', 'euro_capacite', 'euro_abonnement', 'euro_ARENH',
            'euro_CG', 'euro_CC', 'euro_CS_fixe', 'euro_CS_variable', 'euro_CACS',
            'euro_CSPE_TICFE', 'euro_CSPE_TICFE', 'euro_CTA'
        ]
        for attr in necessary_attrs:
            if getattr(self, attr, None) is None:
                print(f"Attribut {attr} non défini.")
                return

        # === Fourniture ===
        fourniture_labels = ['HPB', 'HCB', 'HPH', 'HCH', 'Pointe', 'Capacité', 'Abonnement', 'ARENH (déduction)']
        fourniture_values = [
            self.euro_HPB,
            self.euro_HCB,
            self.euro_HPH,
            self.euro_HCH,
            self.euro_pointe,
            self.euro_capacite,
            self.euro_abonnement,
            self.euro_ARENH
        ]

        # === Distribution (TURPE) ===
        turpe_labels = ['Gestion (CG)', 'Comptage (CC)', 'Soutirage Fixe', 'Soutirage Variable', 'CMDPS', 'Alimentation Secours']
        turpe_values = [
            self.euro_CG,
            self.euro_CC,
            self.euro_CS_fixe,
            self.euro_CS_variable,
            self.euro_mois_CMDPS,
            self.euro_CACS
        ]

        # === Taxes et Contributions ===
        taxes_labels = ['TICFE/CSPE', 'CTA']
        taxes_values = [
            self.euro_CSPE_TICFE,
            self.euro_CTA
        ]

        fig, axs = plt.subplots(1, 3, figsize=figsize)

        def plot_waterfall(ax, labels, values, title):
            cum_values = [0]
            for v in values:
                cum_values.append(cum_values[-1] + v)

            colors = ['green' if v >= 0 else 'red' for v in values]
            for i, (label, val) in enumerate(zip(labels, values)):
                ax.bar(i, val, bottom=cum_values[i], color=colors[i], edgecolor='black')
                y_text = cum_values[i] + val / 2
                ax.text(i, y_text, f"{val:.2f} €", ha='center', va='center', fontsize=9,
                        color='black' if abs(val) > 20 else 'black')

            total = sum(values)

            ax.set_xticks(range(len(labels)))
            ax.set_xticklabels(labels, rotation=45, ha='right')
            ax.axhline(0, color='black', linewidth=0.8)
            ax.set_title(f"{title}\nTotal : {total:.2f} €", fontsize=12)
            ax.grid(axis='y', linestyle='--', alpha=0.5)

        # Tracer les 3 cascades
        plot_waterfall(axs[0], fourniture_labels, fourniture_values, "Fourniture")
        plot_waterfall(axs[1], turpe_labels, turpe_values, "Distribution (TURPE)")
        plot_waterfall(axs[2], taxes_labels, taxes_values, "Taxes et Contributions")

        plt.suptitle(title, fontsize=16, y=0.97)
        plt.tight_layout(rect=[0, 0, 1, 0.93])
        plt.show()


######################Fonctions pour le calcul du TURPE#######################################""
    def calculate_euro_an_CS_fixe(self, coeff, contrat):
        b = coeff.get("b")

        if len(b) == 1:
            # SDT (Sans Différenciation Temporelle) : CS_fixe = a2 × PS_global
            ps_global = getattr(contrat, 'PS_pointe', 0) or getattr(contrat, 'PS_HPH', 0) or 0
            self.euro_an_CS_fixe = round(b[0] * ps_global, 2)
        elif len(b) < 5:
            self.euro_an_CS_fixe = round(
                b[0] * contrat.PS_HPH +
                b[1] * (contrat.PS_HCH - contrat.PS_HPH) +
                b[2] * (contrat.PS_HPB - contrat.PS_HCH) +
                b[3] * (contrat.PS_HCB - contrat.PS_HPB),
                2
            )
        else:
            self.euro_an_CS_fixe = round(
                b[0] * contrat.PS_pointe +
                b[1] * (contrat.PS_HPH - contrat.PS_pointe) +
                b[2] * (contrat.PS_HCH - contrat.PS_HPH) +
                b[3] * (contrat.PS_HPB - contrat.PS_HCH) +
                b[4] * (contrat.PS_HCB - contrat.PS_HPB), 2)
        
            print("euro_an_CS_fixe =",self.euro_an_CS_fixe," = ",b[0],"*", contrat.PS_pointe,"+",b[1], "* (",contrat.PS_HPH, "-", contrat.PS_pointe,")+",b[2], 
                "* (",contrat.PS_HCH, "-" ,contrat.PS_HPH,") +", b[3], "* (",contrat.PS_HPB, "-" ,contrat.PS_HCH,") +",  b[4], "* (",contrat.PS_HCB, "-" ,contrat.PS_HPB,")"         
                )


    def calculate_euro_CS_variable(self, coeff, facture):
        c = coeff.get("c")

        if len(c) == 1:
            # SDT (Sans Différenciation Temporelle) : CS_var = c × kWh_total
            kWh_total = (facture.kWh_pointe + facture.kWh_HPH + facture.kWh_HCH
                         + facture.kWh_HPB + facture.kWh_HCB)
            self.euro_CS_variable_pointe = 0.0
            self.euro_CS_variable_HPH = 0.0
            self.euro_CS_variable_HCH = 0.0
            self.euro_CS_variable_HPB = 0.0
            self.euro_CS_variable_HCB = 0.0
            self.euro_CS_variable = round(c[0] * kWh_total, 2)

        elif len(c) < 5:
            self.euro_CS_variable_HPH = round(c[0] * facture.kWh_HPH, 2)
            self.euro_CS_variable_HCH = round(c[1] * facture.kWh_HCH, 2)
            self.euro_CS_variable_HPB = round(c[2] * facture.kWh_HPB, 2)
            self.euro_CS_variable_HCB = round(c[3] * facture.kWh_HCB, 2)

            self.euro_CS_variable = (
                + self.euro_CS_variable_HPH
                + self.euro_CS_variable_HCH
                + self.euro_CS_variable_HPB
                + self.euro_CS_variable_HCB
            )

        else:
            self.euro_CS_variable_pointe = round(c[0] * facture.kWh_pointe, 2)
            self.euro_CS_variable_HPH = round(c[1] * facture.kWh_HPH, 2)
            self.euro_CS_variable_HCH = round(c[2] * facture.kWh_HCH, 2)
            self.euro_CS_variable_HPB = round(c[3] * facture.kWh_HPB, 2)
            self.euro_CS_variable_HCB = round(c[4] * facture.kWh_HCB, 2)

            self.euro_CS_variable = (
                self.euro_CS_variable_pointe
                + self.euro_CS_variable_HPH
                + self.euro_CS_variable_HCH
                + self.euro_CS_variable_HPB
                + self.euro_CS_variable_HCB
            )

            print("euro_CS_variable_pointe = ",self.euro_CS_variable_pointe, " = round(", c[0], " * ", facture.kWh_pointe, ", 2)"        )
            print("euro_CS_variable_HPH = ",self.euro_CS_variable_HPH, " = round(", c[1], " * ", facture.kWh_HPH, ", 2)"        )
            print("euro_CS_variable_HCH = ",self.euro_CS_variable_HCH, " = round(", c[2], " * ", facture.kWh_HCH, ", 2)"        )
            print("euro_CS_variable_HPB = ", self.euro_CS_variable_HPB, " = round(", c[3], " * ", facture.kWh_HPB, ", 2)"        )
            print("euro_CS_variable_HCB = ", self.euro_CS_variable_HCB, " = round(", c[4], " * ", facture.kWh_HCB, ", 2)"        )
            # print("euro_CS_variable = ",self.euro_CS_variable, " = (",
            #     self.euro_CS_variable_pointe,
            #     " + ",
            #     self.euro_CS_variable_HPH,
            #     " + ",
            #     self.euro_CS_variable_HCH,
            #     " + ",
            #     self.euro_CS_variable_HPB,
            #     " + ",
            #     self.euro_CS_variable_HCB,
            #     ")"
            # )

    def calculate_euro_mois_CMDPS(self, coeff, facture):
        """Calcul de la Composante Mensuelle des Dépassements de Puissance Souscrite.

        HTA DT : CMDPS = Σ 0.15 × k[i] × a2 × √(Σ ΔP²)  (par classe temporelle)
        HTA SDT (indicateur Pmax) : CMDPS = 0.7 × a2 × ΔPmax
        HTA SDT (10 min) : CMDPS = 0.08 × a2 × Σ ΔP²
        BT > 36 kVA : CMDPS = coefficient_CMDPS × heures_depassement
        """
        k = coeff.get("coefficient_CMDPS", 0.04)

        if self.contrat.domaine_tension == "HTA":
            b = coeff.get("b")
            # SDT : utiliser a2_sdt (coefficient CRE pour CMDPS) au lieu de b
            a2_sdt = coeff.get("a2_sdt")
            if a2_sdt is not None and len(b) == 1:
                # SDT avec indicateur de puissance maximale :
                # CMDPS = 0.7 × a2 × ΔPmax (formule CRE TURPE 4, section 6.3.2)
                delta_pmax = max(
                    facture.depassement_PS_pointe, facture.depassement_PS_HPH,
                    facture.depassement_PS_HCH, facture.depassement_PS_HPB,
                    facture.depassement_PS_HCB
                )
                euro_mois_CMDPS = round(0.7 * a2_sdt * delta_pmax, 2)
            else:
                sum_delta_P2 = [
                    facture.depassement_PS_pointe, facture.depassement_PS_HPH,
                    facture.depassement_PS_HCH, facture.depassement_PS_HPB,
                    facture.depassement_PS_HCB
                ]
                euro_mois_CMDPS = round(
                    sum(b[i] * k * (sum_delta_P2[i] ** 0.5) for i in range(len(b))), 2
                )
        elif self.contrat.domaine_tension == "BT > 36 kVA":
            euro_mois_CMDPS = round(k * facture.heures_depassement, 2)
        else:
            euro_mois_CMDPS = 0.0

        return euro_mois_CMDPS

    def calculate_nb_jour(self):
        if isinstance(self.facture.start, str):
            self.facture.start = parse(self.facture.start).date()
        if isinstance(self.facture.end, str):
            self.facture.end = parse(self.facture.end).date()
        try:
            self.nb_jour = (self.facture.end - self.facture.start).days + 1  # Calculate the number of days between start and end
        except:
            self.nb_jour = 0.0

    def calculate_euro_CS_fixe(self):
        self.calculate_nb_jour()  # Appel de la méthode pour calculer le nombre de jours
        self.euro_CS_fixe = round(self.euro_an_CS_fixe * (self.nb_jour / 365.0), 2)
        #print("euro_CS_fixe = ",self.euro_CS_fixe)

    def calculate_euro_CC(self,coeff):
        self.euro_an_CC = coeff.get("euro_an_CC")
        self.calculate_nb_jour()
        # CG et CC sont facturés au 1/12ème mensuel par Enedis
        # On détecte si la période est ~1 mois (28-31 jours) pour utiliser /12
        # sinon on proratise au jour
        if 28 <= self.nb_jour <= 31:
            self.euro_CC = round(self.euro_an_CC / 12.0, 2)
        else:
            self.euro_CC = round(self.euro_an_CC * (self.nb_jour / 365.0), 2)

    def calculate_euro_CG(self,coeff):
        self.euro_an_CG = coeff.get("euro_an_CG")
        self.calculate_nb_jour()
        if 28 <= self.nb_jour <= 31:
            self.euro_CG = round(self.euro_an_CG / 12.0, 2)
        else:
            self.euro_CG = round(self.euro_an_CG * (self.nb_jour / 365.0), 2)

    def calculate_euro_TURPE(self):
        self.calculate_nb_jour()  # Appel de la méthode pour calculer le nombre de jours
        # Imprimez les valeurs qui étaient initialement None pour l'utilisateur
        if self.euro_an_CG is None:
            print("euro_an_CG est None.")
        if self.euro_an_CC is None:
            print("euro_an_CC est None.")
        if self.euro_an_CS_fixe is None:
            print("euro_an_CS_fixe est None.")
        if self.euro_CS_variable is None:
            print("euro_CS_variable est None.")
        if self.euro_mois_CMDPS is None:
            print("euro_mois_CMDPS est None.")
        if self.euro_an_CACS is None:
            print("euro_an_CACS est None.")
        if self.euro_an_CR is None:
            print("euro_an_CR est None.")
        if self.euro_an_CER is None:
            print("euro_an_CER est None.")
        if self.euro_an_CI is None:
            print("euro_an_CI est None.")
        if self.nb_jour is None:
            print("nb_jour est None.")

        # Assurez-vous que toutes les valeurs sont définies ou mises à 0 si elles sont None
        self.euro_an_CG = self.euro_an_CG if self.euro_an_CG is not None else 0
        self.euro_an_CC = self.euro_an_CC if self.euro_an_CC is not None else 0
        self.euro_an_CS_fixe = self.euro_an_CS_fixe if self.euro_an_CS_fixe is not None else 0
        self.euro_CS_variable = self.euro_CS_variable if self.euro_CS_variable is not None else 0
        self.euro_mois_CMDPS = self.euro_mois_CMDPS if self.euro_mois_CMDPS is not None else 0
        self.euro_an_CACS = self.euro_an_CACS if self.euro_an_CACS is not None else 0
        self.euro_an_CR = self.euro_an_CR if self.euro_an_CR is not None else 0
        self.euro_an_CER = self.euro_an_CER if self.euro_an_CER is not None else 0
        self.euro_an_CI = self.euro_an_CI if self.euro_an_CI is not None else 0
        self.nb_jour = self.nb_jour if self.nb_jour is not None else 0

        try:
            self.euro_TURPE= round(self.euro_an_CG * (self.nb_jour / 365.0) + self.euro_an_CC * (self.nb_jour / 365.0) + self.euro_an_CS_fixe * (self.nb_jour / 365.0) + self.euro_CS_variable + self.euro_mois_CMDPS + self.euro_an_CACS * (self.nb_jour / 365.0) + self.euro_an_CR * (self.nb_jour / 365.0) + self.euro_an_CER * (self.nb_jour / 365.0) + self.euro_an_CI * (self.nb_jour / 365.0), 2)
        except:
            self.euro_TURPE = None
        #print("euro_TURPE = ",self.euro_TURPE)
    
    def calculate_euro_an_TURPE(self):
        try:
            self.euro_an_TURPE=round(self.euro_an_CG+self.euro_an_CC+self.euro_an_CS_fixe+12*self.euro_CS_variable+12*self.euro_mois_CMDPS+self.euro_an_CACS+self.euro_an_CR+self.euro_an_CER+self.euro_an_CI,2) #à modifier pour trater le cas de self.euro_mois_CMDPS
            
        except:
            self.euro_an_TURPE=None

    def calculate_euro_CACS(self, coeff):
        euro_cellules = coeff.get("euro_cellules", 0)
        euro_km_aerien = coeff.get("euro_km_aerien", 0)
        euro_km_souterrain = coeff.get("euro_km_souterrain", 0)
        self.euro_an_CACS = round(
            euro_cellules * self.contrat.nb_cellules_secours +
            euro_km_aerien * self.contrat.km_secours_aerien +
            euro_km_souterrain * self.contrat.km_secours_souterrain, 2
        )
        self.euro_CACS= round(self.euro_an_CACS * (self.nb_jour / 365.0), 2)
        # print("euro_CACS = ",self.euro_CACS)
        # print("euro_an_CACS = ",self.euro_an_CACS)

    def calculate_euro_CTA(self,coeff):
        coef_CTA = coeff.get("coef_CTA")
        self.calculate_nb_jour()  # Appel de la méthode pour calculer le nombre de jours
        #print(coef_CTA)
        # euro éligible au CTA
        self.euro_eligible_CTA =(self.euro_CC + self.euro_CS_fixe + self.euro_CG+self.euro_CACS)
        self.euro_CTA = round(self.euro_eligible_CTA * coef_CTA, 6)
        #print("euro_CTA = ",self.euro_CTA)
        self.euro_an_CTA = round(self.euro_CTA /self.nb_jour*365, 2)
    
    def calculate_euro_CSPE_TICFE(self,coeff):
        c_euro_kwh_CSPE_TICFE = coeff.get("c_euro_kwh_CSPE_TICFE")
        self.euro_CSPE_TICFE = round(self.kWh_Total * c_euro_kwh_CSPE_TICFE, 2)
        #print("euro_CSPE_TICFE",self.euro_CSPE_TICFE)

    # def calculate_euro_total(self):
    #     self.calculate_nb_jour()  # Appel de la méthode pour calculer le nombre de jours
    #     try:
    #         self.euro_total = round(self.euro_HPB + self.euro_HCB + self.euro_HPH + self.euro_HCH + self.euro_pointe + self.euro_ENR + self.euro_ARENH + self.euro_TURPE + self.euro_CTA + self.euro_CSPE_TICFE + self.euro_CSPE_TICFE + self.euro_capacite_pointe + self.euro_capacite_HPH, 2)
    #     except:
    #         self.euro_total = None
    #     print("euro_total = ",self.euro_total)

    def calculate_montant(self, coeff):

        # self.kWh_Total = self.facture.kWh_HPH + self.facture.kWh_HCH + self.facture.kWh_HPB + self.facture.kWh_HCB + self.facture.kWh_pointe

        self.kWh_ENR = (self.contrat.pourcentage_ENR / 100) * self.kWh_Total if self.contrat.pourcentage_ENR and self.kWh_Total else 0.0
        #print("kWh_ENR = ",self.kWh_ENR," = ","(",self.contrat.pourcentage_ENR," / 100) * ",self.kWh_Total)
        self.euro_pointe = round(self.facture.kWh_pointe * self.tarif.c_euro_kWh_pointe, 2)
        #print("euro_pointe = ",self.euro_pointe)
        self.euro_HPB = round(self.facture.kWh_HPB * self.tarif.c_euro_kWh_HPB, 2)
        #print("euro_HPB = ",self.euro_HPB)
        self.euro_HCB = round(self.facture.kWh_HCB * self.tarif.c_euro_kWh_HCB, 2)
        #print("euro_HCB = ",self.euro_HCB)
        self.euro_HPH = round(self.facture.kWh_HPH * self.tarif.c_euro_kWh_HPH, 2)
        #print("euro_HPH = ",self.euro_HPH)
        self.euro_HCH = round(self.facture.kWh_HCH * self.tarif.c_euro_kWh_HCH, 2)
        #print("euro_HCH = ",self.euro_HCH)

        if self.tarif.c_euro_kwh_CSPE_TICFE is None:
            self.tarif.c_euro_kwh_CSPE_TICFE=coeff.get("c_euro_kwh_CSPE_TICFE")

        self.euro_CSPE_TICFE = round(self.kWh_Total * self.tarif.c_euro_kwh_CSPE_TICFE, 2)
        self.euro_an_CSPE_TICFE = round(self.euro_CSPE_TICFE / self.nb_jour * 365, 2) if self.nb_jour else None
        #print("euro_CSPE_TICFE = ",self.euro_CSPE_TICFE)
        try:
            self.euro_capacite_pointe = round(self.facture.kWh_pointe * self.tarif.c_euro_kWh_certif_capacite_pointe, 2)
            #print("euro_capacite_pointe = ",self.euro_capacite_pointe)
        except:
            pass
        self.euro_capacite_HPH = round(self.facture.kWh_HPH * self.tarif.c_euro_kWh_certif_capacite_HPH, 2)
        #print("euro_capacite_HPH = ",self.euro_capacite_HPH)
        self.euro_capacite_HCH = round(self.facture.kWh_HCH * self.tarif.c_euro_kWh_certif_capacite_HCH, 2)
        #print("euro_capacite_HCH = ",self.euro_capacite_HCH)
        self.euro_capacite_HPB = round(self.facture.kWh_HPB * self.tarif.c_euro_kWh_certif_capacite_HPB, 2)
        #print("euro_capacite_HPB = ",self.euro_capacite_HPB)
        self.euro_capacite_HCB = round(self.facture.kWh_HCB * self.tarif.c_euro_kWh_certif_capacite_HCB, 2)
        #print("euro_capacite_HCB = ",self.euro_capacite_HCB)

        self.euro_capacite = (self.euro_capacite_pointe + self.euro_capacite_HPH + self.euro_capacite_HCH + self.euro_capacite_HPB + self.euro_capacite_HCB)
        #print("euro_capacite = ",self.euro_capacite)
        self.euro_ENR = 0 if (self.kWh_Total is None or self.tarif.c_euro_kWh_ENR is None) else round(self.kWh_Total * self.tarif.c_euro_kWh_ENR, 2)
        # self.euro_ENR = round(self.kWh_Total * self.tarif.c_euro_kWh_ENR, 2)
        #print("euro_ENR = ",self.euro_ENR)
        self.euro_ARENH = round(self.kWh_Total * self.tarif.c_euro_kWh_ARENH, 2)
        #print("euro_ARENH  = ",self.euro_ARENH)
        self.euro_abonnement= self.contrat.abonnement_annuel/365* self.nb_jour if self.contrat.abonnement_annuel and self.nb_jour else 0.0
        #print("euro_abonnement = ",self.euro_abonnement)
        self.euro_fourniture = round(self.euro_HPB + self.euro_HCB + self.euro_HPH + self.euro_HCH + self.euro_pointe + self.euro_ENR + self.euro_ARENH+self.euro_capacite+self.euro_abonnement, 2)
        #print("euro_fourniture  = ",self.euro_fourniture)

        try:
            self.euro_an_TURPE=round(self.euro_an_CG+self.euro_an_CC+self.euro_an_CS_fixe+12*self.euro_CS_variable+12*self.euro_mois_CMDPS+self.euro_an_CACS+self.euro_an_CR+self.euro_an_CER+self.euro_an_CI,2)
        except:
            self.euro_an_TURPE = None
        #print("euro_an_TURPE = ",self.euro_an_TURPE)

        try: 
            self.euro_total = round(self.euro_HPB + self.euro_HCB + self.euro_HPH + self.euro_HCH + self.euro_pointe + self.euro_ENR + self.euro_ARENH + self.euro_TURPE + self.euro_CTA + self.euro_CSPE_TICFE + self.euro_capacite + self.euro_abonnement, 2)
        except:
            self.euro_total = None
        #print(self.euro_HPB ,"+", self.euro_HCB ,"+", self.euro_HPH ,"+", self.euro_HCH ,"+", self.euro_pointe ,"+", self.euro_ENR ,"+", self.euro_ARENH ,"+", self.euro_TURPE ,"+", self.euro_CTA ,"+", self.euro_CSPE_TICFE ,"+", self.euro_capacite_pointe ,"+", self.euro_capacite_HPH)
        #print("euro_total = ",self.euro_total)


   



#from .TkinterGUI import TkinterGUI
from .FluidPort import FluidPort
from .Expansion_Valve import Expansion_Valve
from .Evaporator import Evaporator
from .Desuperheater import Desuperheater
from .Condenser import Condenser 
from .Compressor import Compressor
from .Pump import Pump
#from .Turbine import Turbine
#from .HEX import Simple_HEX
from .Tank import MixedStorage




from CoolProp.CoolProp import PropsSI
import pandas as pd

class FluidPort:
    def __init__(self, fluid='ammonia', P=None, h=None, F=None, T=None, S=None):
        self._F = F
        self._P = P
        self.h = h
        self.fluid = fluid
        self.T = T
        self.S = S
        self.rho = None
        self.cp = None
        self.lamda = None
        self.mu = None
        self.callback = None
        # Pour la gestion des nœuds hydrauliques
        self.connected_inlets = []  # Liste des inlets connectés à ce port (pour outlets)
        self.connected_outlets = []  # Liste des outlets connectés à ce port (pour inlets)
        self.node_id = None  # ID du nœud hydraulique si applicable
        self.calculate_properties()

    @property
    def F(self):
        """Getter pour le débit massique."""
        return self._F
    
    @F.setter
    def F(self, value):
        """Setter pour le débit massique avec détection de changement."""
        if self._F != value:  # Vérifie si la valeur a changé
            self._F = value
            print(f"FluidPort.F a changé : {value} kg/s")
            if self.callback:  # Si un callback est défini, l'appeler
                self.callback()

    @property
    def P(self):
        """Getter pour la pression."""
        return self._P

    @P.setter
    def P(self, value):
        """Setter pour la pression avec détection de changement."""
        if self._P != value:  # Vérifie si la valeur a changé
            self._P = value
            print(f"FluidPort.P a changé : {value}")
            self.calculate_properties()  # Recalcule les propriétés du fluide
            if self.callback:  # Si un callback est défini, l'appeler
                self.callback()

    def propriete(self, Pro, I1, ValI1, I2, ValI2):
        result = PropsSI(Pro, I1, ValI1, I2, ValI2, self.fluid)
        return result

    def calculate_properties(self):
        # Ensure all needed properties are set

        if self.P is not None and self.T is not None and self.h is None:
            self.h = PropsSI('H', 'T', self.T, 'P', self.P, self.fluid)
            #print("self.h",self.h)
        
        if self.P is not None and self.h is not None:
            #print("self.....P",self.P)
            self.T = PropsSI('T', 'P', self.P, 'H', self.h, self.fluid)
            self.S = PropsSI('S', 'P', self.P, 'H', self.h, self.fluid)
            self.rho = PropsSI('D', 'P', self.P, 'H', self.h, self.fluid)
            self.cp = PropsSI('C', 'P', self.P, 'H', self.h, self.fluid)
            self.lamda = PropsSI('L', 'P', self.P, 'H', self.h, self.fluid)
            self.mu = PropsSI('V', 'P', self.P, 'H', self.h, self.fluid)

     
    
        self.df = pd.DataFrame({
            'FluidPort': [self.fluid, self.F, self.T, self.P, self.h, self.S, self.rho, self.cp, self.lamda, self.mu],
            },
            index=['Fluide', 'Débit(kg/s)', 'Température(k)', 'Pression (Pa)', 'Enthalpie (J/kg)', 'Entropie (J/kg-K)',
                   'Densité (kg/m^3)', 'Chaleur spécifique (J/kg-K)', 'Conductivité thermique (W/m-K)', 'Viscosité dynamique (Pa-s)'])



import pandas as pd
import numpy as np


from .functions.plotting import plot_streams_and_temperature_intervals
from .functions.plotting import plot_GCC
from .functions.plotting import plot_composites_curves
from .functions.calculation_decomposition import decomposition_flux
from .functions.calculation_surplus_deficit import surplus_deficit
from .functions.calculation_composite_curve import composite_curve
from .functions.calculation_below_above_stream import below_above_stream
from .functions.calculation_allowed_combinations import find_heat_exchange_combinations
from .functions.calculation_HEN import calculate_stream_numbers, hen_stream_list, HeatExchangerNetwork
from .functions.calculation_graphical_HEN import graphical_hen_design

class Object:
    def __init__(self, df):
        #initialisation 


        # Créer la colonne 'integration' et la remplir avec True si elle n'existe pas
        if 'integration' not in df.columns:
            df['integration'] = True
        # Remplacer les valeurs NaN par True
        df['integration'].fillna(True, inplace=True)

        # Sélectionner les flux à intégrer
        self.stream_list = df[df['integration'] == True].copy()  # Utilisez .copy() pour éviter le Warning

        self.rowCount = len(self.stream_list)

        # Créer la colonne 'StreamType'
        self.stream_list['StreamType'] = np.where(self.stream_list['Ti'] > self.stream_list['To'], 'HS', 'CS')

        # Créer de nouvelles colonnes pour les températures décalées
        self.stream_list['STi'] = np.where(self.stream_list['StreamType'] == 'HS',
                                                  self.stream_list['Ti'] - self.stream_list['dTmin2'],
                                                  self.stream_list['Ti'] + self.stream_list['dTmin2'])

        self.stream_list['STo'] = np.where(self.stream_list['StreamType'] == 'HS',
                                                  self.stream_list['To'] - self.stream_list['dTmin2'],
                                                  self.stream_list['To'] + self.stream_list['dTmin2'])

        self.stream_list['delta_H'] = self.stream_list['mCp'] * (self.stream_list['To'] - self.stream_list['Ti'])

         # Calculer T_shifted directement dans la classe
        T_shifted = np.concatenate([self.stream_list['STi'].values, self.stream_list['STo'].values])
        T_shifted = np.sort(np.unique(T_shifted))[::-1]
        self.df_T_shifted = pd.DataFrame({'T_shifted': T_shifted})



                # Créer le DataFrame df_intervals
        self.df_intervals = pd.DataFrame({'Tsup': T_shifted[:-1], 'Tinf': T_shifted[1:]})
        self.df_intervals['IntervalName'] = self.df_intervals['Tsup'].astype(str) + '-' + self.df_intervals['Tinf'].astype(str)

        self.decomposition_flux()
        self.surplus_deficit()
        self.composite_curve()
        self.below_above_stream()
        self.find_heat_exchange_combinations()
        self.calculate_stream_numbers()
        self.hen_stream_list()



    decomposition_flux=decomposition_flux

    plot_streams_and_temperature_intervals = plot_streams_and_temperature_intervals
    plot_GCC=plot_GCC
    plot_composites_curves=plot_composites_curves
    surplus_deficit=surplus_deficit
    composite_curve=composite_curve
    below_above_stream=below_above_stream

# #####################################################""""

#########################################################################################"

    find_heat_exchange_combinations=find_heat_exchange_combinations

###################"For Heat Exchanger network"
    calculate_stream_numbers=calculate_stream_numbers
    hen_stream_list=hen_stream_list
    graphical_hen_design=graphical_hen_design
    HeatExchangerNetwork=HeatExchangerNetwork



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `nodeeditor` package."""


import unittest

from nodeeditor.node_scene import Scene


class TestTemplate(unittest.TestCase):
    """Tests for `nodeeditor` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_000_something(self):
        """Test if Scene got has_been_modified property."""
        assert(hasattr(Scene, 'has_been_modified'))
